#########################################################################
#
# Date: Apr 2006 Authors: Ruth Huey, Garrett Morris
#
#    rhuey@scripps.edu
#    garrett@scripps.edu
#
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Ruth Huey, Garrett Morris and TSRI
#
#########################################################################

import warnings

from Vision import UserLibBuild
from NetworkEditor.items import NetworkNode
from MolKit.molecule import Atom, AtomSet, Molecule, MoleculeSet
from MolKit.protein import Residue, ResidueSet, Chain, ChainSet
from AutoDockTools.atomTypeTools import AutoDock4_AtomTyper
from AutoDockTools.atomTypeTools import NonpolarHydrogenMerger, LonepairMerger
from AutoDockTools.atomTypeTools import AromaticCarbonManager, SolvationParameterizer
from AutoDockTools.MoleculePreparation import RotatableBondManager
from AutoDockTools.MoleculePreparation import LigandWriter, AD4LigandWriter



def importAdtLib(net):
    try:
        from AutoDockTools.VisionInterface.AdtNodes import adtlib
        net.editor.addLibraryInstance(
            adtlib, 'AutoDockTools.VisionInterface.AdtNodes', 'adtlib')
    except:
        warnings.warn(
            'Warning! Could not import adtlib from AutoDockTools.VisionInterface')
            

class GridParameterFileBrowserNE(NetworkNode):
    """A specialized Tkinter Filebrowser. Double-clicking into the entry opens the
filebrowser."""
    
    def __init__(self, name='Grid Parameter File Browser', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )

        #self.readOnly = 1
        code = """def doit(self, filename):
    if filename:
        self.outputData(filename=filename)
"""

        self.setFunction(code)

        # show the entry widget by default
        self.inNodeWidgetVisibleByDefault = True

        fileTypes=[('gpf', '*')]

        self.widgetDescr['filename'] = {
            'class':'NEEntryWithFileBrowser', 'master':'node',
            'filetypes':fileTypes, 'title':'read file', 'width':16,
            'labelCfg':{'text':'gpf file:'},
            }


        #self.widgetDescr['filename'] = {
        #    'class':'NEEntryWithFileBrowser', 'master':'node', 'width':16,
        #    'initialValue':'', 'lockedOnPort':True, 
        #    'labelCfg':{'text':'Filename: '}
        #    }

        self.inputPortsDescr.append(datatype='string', name='filename')

        self.outputPortsDescr.append(datatype='string', name='filename')


class DockingParameterFileBrowserNE(NetworkNode):
    """A specialized Tkinter Filebrowser. Double-clicking into the entry opens the
filebrowser."""
    
    def __init__(self, name='Docking Parameter File Browser', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )

        #self.readOnly = 1
        code = """def doit(self, filename):
    if filename:
        self.outputData(filename=filename)
"""

        self.setFunction(code)

        # show the entry widget by default
        self.inNodeWidgetVisibleByDefault = True

        fileTypes=[('dpf', '*')]

        self.widgetDescr['filename'] = {
            'class':'NEEntryWithFileBrowser', 'master':'node',
            'filetypes':fileTypes, 'title':'read file', 'width':16,
            'labelCfg':{'text':'dpf file:'},
            }

        self.inputPortsDescr.append(datatype='string', name='filename')

        self.outputPortsDescr.append(datatype='string', name='filename')



class ReadGridParameterFile(NetworkNode):
    #mRequiredTypes = {}
    #mRequiredSynonyms = [
    #]
    def __init__(self, constrkw = {},  name='ReadGridParameterFile', **kw):
        kw['constrkw'] = constrkw
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw)
        fileTypes=[('gpf', '*')]

        self.widgetDescr['filename'] = {
            'class':'NEEntryWithFileBrowser', 'master':'node',
            'filetypes':fileTypes, 'title':'read file', 'width':16,
            'labelCfg':{'text':'file:'},
            }

        code = """def doit(self, template_gpf_filename):
    if template_gpf_filename:
            from AutoDockTools.GridParameters import GridParameters
            gpo = GridParameters()
            gpo.read(template_gpf_filename)
            self.outputData(gpo=gpo)
"""
        self.configure(function=code)

        self.inputPortsDescr.append(
            {'name': 'template_gpf_filename', 'cast': True, 'datatype': 'string', 'balloon': 'template grid parameter filename', 'required': False, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
        self.outputPortsDescr.append(
            {'name': 'gpo', 'datatype': 'None', 'balloon': 'gpo,  grid parameter object,  instance of AutoDockTools.GridParameters', 'height': 8, 'width': 12, 'shape': 'diamond', 'color': 'white'})
        self.widgetDescr['template_gpf_filename'] = {
            'initialValue': '', 'labelGridCfg': {'column': 0, 'row': 0}, 'master': 'node', 'widgetGridCfg': {'labelSide': 'left', 'column': 1, 'row': 0}, 'labelCfg': {'text': ''}, 'class': 'NEEntryWithFileBrowser'}

    def beforeAddingToNetwork(self, net):
        try:
            ed = net.getEditor()
        except:
            import traceback; traceback.print_exc()
            print 'Warning! Could not import widgets'



class ReadDockingParameterFile(NetworkNode):
    #mRequiredTypes = {}
    #mRequiredSynonyms = [
    #]
    def __init__(self, constrkw = {},  name='ReadDockingParameterFile', **kw):
        kw['constrkw'] = constrkw
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw)
        fileTypes=[('dpf', '*')]

        self.widgetDescr['filename'] = {
            'class':'NEEntryWithFileBrowser', 'master':'node',
            'filetypes':fileTypes, 'title':'read file', 'width':16,
            'labelCfg':{'text':'file:'},
            }

        code = """def doit(self, template_dpf_filename):
    if template_dpf_filename:
            from AutoDockTools.DockingParameters import DockingParameters
            gpo = DockingParameters()
            gpo.read(template_dpf_filename)
            self.outputData(gpo=gpo)
"""
        self.configure(function=code)

        self.inputPortsDescr.append(
            {'name': 'template_dpf_filename', 'cast': True, 'datatype': 'string', 'balloon': 'template grid parameter filename', 'required': False, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
        self.outputPortsDescr.append(
            {'name': 'gpo', 'datatype': 'None', 'balloon': 'gpo,  grid parameter object,  instance of AutoDockTools.DockingParameters', 'height': 8, 'width': 12, 'shape': 'diamond', 'color': 'white'})
        self.widgetDescr['template_dpf_filename'] = {
            'initialValue': '', 'labelGridCfg': {'column': 0, 'row': 0}, 'master': 'node', 'widgetGridCfg': {'labelSide': 'left', 'column': 1, 'row': 0}, 'labelCfg': {'text': ''}, 'class': 'NEEntryWithFileBrowser'}

    def beforeAddingToNetwork(self, net):
        try:
            ed = net.getEditor()
        except:
            import traceback; traceback.print_exc()
            print 'Warning! Could not import widgets'


###class ReadGridParameterFile(NetworkNode):
###    """Read a Grid Parameter file [using Python's readlines() command.]
###Double-clicking on the node opens a text entry widget to type the file name.
###In addition, double-clicking in the text entry opens a file browser window.

###Input Ports
###    filename: name of the file to be opened

###Output Ports
###    data: a list of strings
###"""

###    def __init__(self, name='Read Parameter File', **kw):
###        kw['name'] = name
###        apply( NetworkNode.__init__, (self,), kw )

###        #self.readOnly = 1
###        code = """def doit(self, filename):
###    if filename and len(filename):
###        gpo = GridParameters()
###        gpo.read(filename)
###        #f = open(filename)
###        #datastream = f.readlines()
###        #f.close()
###        #if datastream:
###        self.outputData(data=gpo)
###"""

###        self.setFunction(code)

###        fileTypes=[('gpf', 'gpf')]

###        self.widgetDescr['filename'] = {
###            'class':'NEEntryWithFileBrowser', 'master':'node',
###            'filetypes':fileTypes, 'title':'read file', 'width':16,
###            'labelCfg':{'text':'file:'},
###            }


###        self.inputPortsDescr.append(datatype='string', name='filename')
###        self.outputPortsDescr.append(datatype='instance', name='gpo')


class RemoveWaters(NetworkNode):
    """ removes water residues
Process entails:
#    1.  looping over each molecule in input
#    2.  selecting all atoms in water residues
#    3.  removing all bonds from each atom
#    4.  removing each atom from its parent residue
#    5.  removing parent residue if it has no remaining atoms
#    6.  removing parent chain if it has no remaining residues
#    7.  resetting allAtoms attribute of molecule
#    8.  returning number of water residues removed
Input:  molecules (MoleculeSet)
Output: molecules where all atoms in HOH residues have been removed(MoleculeSet)"""
    
    def __init__(self, name='RemoveWaters', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )
        ip = self.inputPortsDescr
        ip.append(datatype='MoleculeSet', name='molecules')
        ip.append(datatype='str', required=False, name='residue_type_str', defaultValue='HOH')
        op = self.outputPortsDescr
        op.append(datatype='MoleculeSet', name='molecules_with_no_water_residues')
        op.append(datatype='int', name='num_water_res')

        code = """def doit(self, molecules, residue_type_str):
    if molecules:
        from MolKit.molecule import BondSet
        lenHOHs = 0
        for mol in molecules:
            hohs = mol.allAtoms.get(lambda x: x.parent.type==residue_type_str)
            if hohs:
                #remove(hohs)
                lenHOHs = len(hohs)
                for h in hohs:
                    for b in h.bonds:
                        c = b.atom1
                        if c==h:
                            c = b.atom2
                        c.bonds.remove(b)
                    h.bonds = BondSet()
                    res = h.parent
                    h.parent.remove(h)
                    if len(h.parent.children)==0:
                        res = h.parent
                        chain = res.parent
                        #print 'removing residue: ', res.name
                        chain.remove(res)
                        if len(chain.children)==0:
                            mol = chain.parent
                            print 'removing chain', chain.id
                            mol.remove(chain)
                            del chain
                        del res
                    del h 
                #fix allAtoms short cut
                mol.allAtoms = mol.chains.residues.atoms    
    self.outputData(molecules_with_no_water_residues=molecules, num_water_res=lenHOHs)"""
        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass




class MergeNonPolarHydrogens(NetworkNode):
    """ merges nonpolar hydrogens 
Process entails:
#    1.  adding charge on nonpolar hydrogen to charge of carbon atom to which it is bonded
#    2.  removing the nonpolar hydrogen from the molecule
#    3.  resetting allAtoms attribute of molecule
#    4.  returning number of nonpolar hydrogens removed
Input:  mols (MoleculeSet)
Output: mols where each non-polar hydrogen atom has been removed(MoleculeSet)"""
    
    def __init__(self, name='Nonpolar Hydrogen Merger', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )
        ip = self.inputPortsDescr
        ip.append(datatype='MoleculeSet', name='mols')
        ip.append(datatype='int', required=False, name='renumber', defaultValue=1)

        op = self.outputPortsDescr
        op.append(datatype='MoleculeSet', name='mols')
        op.append(datatype='int', name='num_nphs')

        code = """def doit(self,mols, renumber):
    if mols:
        nph_merger = NonpolarHydrogenMerger()
        num_nphs = 0
        for mol in mols:
            if not len(mol.allAtoms.bonds[0]):
                mol.buildBondsByDistance()
            num_nphs = nph_merger.mergeNPHS(mol.allAtoms, renumber=renumber)
        self.outputData(mols=mols, num_nphs=num_nphs)\n"""

        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass



class  MergeLonePairs(NetworkNode):
    """ merges lone pairs
Process entails:
#    1.  adding charge on lone pair 'atom' to charge of carbon atom to which it is 'bonded'
#    2.  removing the lone pair 'atom' from the molecule
#    3.  resetting allAtoms attribute of molecule
#    4.  returning number of lone pair 'atoms' removed
Input:  mols (MoleculeSet)
Output: mols where each lone pair 'atom' has been removed(MoleculeSet)"""
    
    def __init__(self, name='Lone Pair Merger', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )
        ip = self.inputPortsDescr
        ip.append(datatype='MoleculeSet', name='mols')
        ip.append(datatype='int', required=False, name='renumber', defaultValue=1)

        op = self.outputPortsDescr
        op.append(datatype='MoleculeSet', name='mols')
        op.append(datatype='int', name='num_lps')

        code = """def doit(self,mols, renumber):
    if mols:
        lps_merger = LonepairMerger()
        num_lps = 0
        for mol in mols:
            if not len(mol.allAtoms.bonds[0]):
                mol.buildBondsByDistance()
            lps = lps_merger.mergeLPS(mol.allAtoms, renumber=renumber)
            num_lps += len(lps)
        self.outputData(mols=mols, num_lps=num_lps)\n"""

        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass


class  ManageAromaticCarbons(NetworkNode):
    """ manages assigning autodock carbon atom types: aliphatic and aromatic
Process entails:
#    1.  'setAromaticCarbons' method detects cyclic aromatic carbons: renaming 
#    them 'A' and setting autodock_element to 'A'. Cyclic carbons are 'aromatic' 
#    if the angle between adjacent normals  is  less than a specified 'cutoff' 
#    angle which is 7.5 degrees by default. Returns atoms which are changed 
#    2.  provides widget for changing the 'cutoff'
#    3.  NOT_IMPLEMENTED:provides method 'set_carbon_names' for forcing any 'C' to 'A' and
#    the opposite. Returns atoms which are changed
Input:  mols (MoleculeSet)
Output: mols where aromatic carbons have been detected (MoleculeSet)"""
    
    def __init__(self, name='Manage Aromatic Carbons', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )

        self.widgetDescr['cutoff'] = {
                'class':'NEDial', 'size':50,
                'oneTurn':5.0, 'min':1.0, 'lockMin':1, 'type':'float',
                'initialValue':7.5,
                'labelGridCfg':{'sticky':'w'},
                'labelCfg':{'text':'cutoff'},
                }

        ip = self.inputPortsDescr
        ip.append(datatype='MoleculeSet', name='mols')
        ip.append(datatype='float', required=False, name='cutoff', defaultValue=7.5)

        op = self.outputPortsDescr
        op.append(datatype='MoleculeSet', name='mols')
        op.append(datatype='int', name='num_aromaticCs')

        code = """def doit(self, mols, cutoff):
    if mols:
        aromC_manager = AromaticCarbonManager(cutoff=cutoff)
        num_aromCs = 0
        for mol in mols:
            if not len(mol.allAtoms.bonds[0]):
                mol.buildBondsByDistance()
            aromCs = aromC_manager.setAromaticCarbons(mol, cutoff=cutoff)
            num_aromCs+=len(aromCs)
        self.outputData(mols=mols, num_aromaticCs=num_aromCs)\n"""

        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass


class  Assign_AD4Types(NetworkNode):
    """ assigns autodock4-style 'autodock_element' to atoms
Process entails:
#    1.  distinguishing between nitrogens which do accept hydrogen-bonds and
#    those which do not (because they already have bonds to hydrogens)
#    2.  distinguishing between oxygens which do accept hydrogen-bonds and
#    those which do not (because they already have bonds to hydrogens)
#    3.  setting autodock_element to 'A' for carbon atoms in cycles in standard amino acids
#    4.  setting autodock_element to 'HD' for all hydrogen atoms   
#    5.  setting autodock_element for all other atoms to the atom's element
#    NOTE: more types can be added if more distinctions are supported by
#    autodock
Input:  mols (MoleculeSet)
Output: typed_mols where each atom has autodock_element field(MoleculeSet)"""
    
    def __init__(self, name='AD4_typer', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )
        ip = self.inputPortsDescr
        ip.append(datatype='MoleculeSet', name='mols')
        ip.append(datatype='int', required=False, name='set_aromatic_carbons', defaultValue=1)
        ip.append(datatype='int', required=False, name='reassign', defaultValue=1)
        ip.append(datatype='int', required=False, name='renameAtoms', defaultValue=0)

        op = self.outputPortsDescr
        op.append(datatype='MoleculeSet', name='typed_mols')

        code = """def doit(self,mols, set_aromatic_carbons=1, reassign=1, renameAtoms=0):
    if mols:
        at_typer = AutoDock4_AtomTyper(set_aromatic_carbons=1, renameAtoms=renameAtoms)
        for mol in mols:
            if not len(mol.allAtoms.bonds[0]):
                mol.buildBondsByDistance()
            at_typer.setAutoDockElements(mol, reassign=reassign)
        self.outputData(typed_mols=mols)\n"""

        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass



class  Add_SolvationParameters(NetworkNode):
    """ assigns autodock3-style 'solvation parameters' to atoms
Process entails:
#    1.  distinguishing between aromatic and aliphatic carbons
#    2.  look-up table of solvation volumes (AtSolVol)
#    3.  setting AtSolPar and AtVol
Input:  mols (MoleculeSet)
Output: typed_mols where each atom has SolVol and AtSolPar fields(MoleculeSet)"""
    
    def __init__(self, name='AD4_SolvationParameterizer', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )
        ip = self.inputPortsDescr
        ip.append(datatype='MoleculeSet', name='mols')

        op = self.outputPortsDescr
        op.append(datatype='MoleculeSet', name='typed_mols')
        op.append(datatype='list', name='AtSolPar')

        code = """def doit(self,mols):
    if mols:
        SP = SolvationParameterizer()
        for mol in mols:
            unknown_atoms = SP.addParameters(mol.chains.residues.atoms)
            #?keep information about unknown_atoms?
            if unknown_atoms is not None: mol.unknown_atoms = len(unknown_atoms)
        self.outputData(typed_mols=mols, AtSolPar=mols.allAtoms.AtSolPar)\n"""
        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass



class  ManageRotatableBonds(NetworkNode):
    """ manages setting flexibility pattern in molecule
Process entails:
#    1.  distinguishing between possibly-rotatable and non-rotatable bonds
#    2.  turning on/off classes of possibly-rotatable bonds such as 
#           amide, guanidinium, peptide-backbone
#    3.  optionally turning on/off specific bonds between named atoms
#    4.  optionally limiting the total number of rotatable bonds 
#           toggling either: 
#               those which move the most atoms
#                   or 
#               those which move the fewest atoms
Input:  mol (Molecule)
Output: mol with marked bonds """
    
    def __init__(self, name='Manage Rotatable Bonds', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )
        ip = self.inputPortsDescr
        ip.append(datatype='MoleculeSet', name='mols', defaultValue='auto')
        ip.append(datatype='string', required=False, name='root')
        ip.append(datatype='string', required=False, name='allowed_bonds', defaultValue='backbone')
        ip.append(datatype='int', required=False, name='check_for_fragments', defaultValue=0)
        ip.append(datatype='string', required=False, name='bonds_to_inactivate', defaultValue='')
        ip.append(datatype='string', required=False, name='limit_torsions', defaultValue='')

        op = self.outputPortsDescr
        op.append(datatype='MoleculeSet', name='mols')

        code = """def doit(self, mols, root, allowed_bonds, check_for_fragments,
bonds_to_inactivate, limit_torsions):
    if mols:
        #mol = mols[0]
        for mol in mols:
            if not len(mol.allAtoms.bonds[0]):
                mol.buildBondsByDistance()
            print "root=", root
            mol.RBM = RotatableBondManager(mol, allowed_bonds, root,
                            check_for_fragments=check_for_fragments,
                            bonds_to_inactivate=bonds_to_inactivate)
            if limit_torsions:
                mol.RBM.limit_torsions(limit_torsions)
        self.outputData(mols=mols)\n"""

        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass



class  Ligand_Writer(NetworkNode):
    """ writes autodock3 ligand pdbq file
Process entails:
#    1.  writing REMARK records about torsions
#    2.  writing ROOT/ENDROOT records about rigid portion of ligand
#    3.  writing BRANCH/ENDBRANCH records about movable sections of ligand
#    4.  writing TORSDOF record showing torsional degrees of freedom
Input:  mol (Molecule), output_filename (string)
Output: mol """
    
    def __init__(self, name='Ligand_Writer', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )

        fileTypes=[('pdbq', '*.pdbq'), ('all', '*')]

        self.widgetDescr['output_filename'] = {
            'class':'NEEntryWithFileSaver', 'master':'node',
            'filetypes':fileTypes, 'title':'save AD3 Ligand', 'width':10,
            'labelCfg':{'text':'file:'},
            }
        
        ip = self.inputPortsDescr
        ip.append(datatype='Molecule', name='mol')
        ip.append(datatype='string',  name='output_filename')

        op = self.outputPortsDescr
        op.append(datatype='Molecule', name='mol')

        code = """def doit(self, mol, output_filename):
    if mol:
        #mol = mols[0]
        #check for bonds with 'possibleTors/activeTOrs keys'
        #check for root
        #check for TORSDOF
        #check for charges
        writer = LigandWriter()
        writer.write(mol, output_filename)
        self.outputData(mol=mol)\n"""

        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass



class  Ligand_Writer_AD4(NetworkNode):
    """ writes autodock4 ligand pdbqt file
Process entails:
#    1.  writing REMARK records about torsions
#    2.  writing ROOT/ENDROOT records about rigid portion of ligand
#    3.  writing BRANCH/ENDBRANCH records about movable sections of ligand
#    4.  writing TORSDOF record showing torsional degrees of freedom
Input:  mol (Molecule), output_filename (string)
Output: mol, output_filename """
    
    def __init__(self, name='Ligand_Writer_AD4', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )

        fileTypes=[('pdbqt', '*.pdbqt'), ('all', '*')]

        self.widgetDescr['output_filename'] = {
            'class':'NEEntryWithFileSaver', 'master':'node',
            'filetypes':fileTypes, 'title':'save AD4 Ligand', 'width':10,
            'labelCfg':{'text':'file:'},
            }
        
        ip = self.inputPortsDescr
        ip.append(datatype='Molecule', name='mol')
        ip.append(datatype='string', name='output_filename')

        op = self.outputPortsDescr
        op.append(datatype='Molecule', name='mol')
        op.append(datatype='string', name='output_filename')

        code = """def doit(self, mol, output_filename):
    if mol:
        writer = AD4LigandWriter()
        writer.write(mol, output_filename)
        self.outputData(mol=mol, output_filename=output_filename)\n"""
        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass



####class AdtPrepareLigand(NetworkNode):
####    """ formats ligand for autodock3 using AutoDockTools.MoleculePreparation.LigandPreparation.
####Process entails:
####    1. possible clean_up:
####            removing lone pairs
####            merging non_polar hydrogens
####            adding bonds to atoms with no bonds
####    2. making atoms in ligand conform to autodock3 atom types:
####            distinction between carbons and cyclic-aromatic carbons, 
####            no non-polar hydrogens
####    3. adding partial charges (gasteiger by default)
####    4. establishing 'torTree' for flexibility pattern by setting 'root' and 'rotatable' bonds
####    5. writing 'pdbq' file
####Input:  mols (MoleculeSet)
####Output: AD3ligand (Molecule)"""


###class Adt4PrepareLigand(NetworkNode):
###    """ formats ligand for autodock4 using AutoDockTools.MoleculePreparation.AD4LigandPreparation.
###Process entails:
###    1. possible clean_up:
###            removing lone pairs
###            merging non_polar hydrogens
###            adding bonds to atoms with no bonds
###    2. making atoms in ligand conform to autodock3 atom types:
###            distinction between carbons and cyclic-aromatic carbons, 
###            no non-polar hydrogens
###    3. adding partial charges (gasteiger by default)
###    4. establishing 'torTree' for flexibility pattern by setting 'root' and 'rotatable' bonds
###    5. writing 'pdbqt' file
###Input:  mols (MoleculeSet)
###Output: AD4ligand (Molecule)"""


###class AdtPrepareReceptor(NetworkNode):
###    """ formats receptor for autodock3 using AutoDockTools.MoleculePreparation.ReceptorPreparation.
###Process entails:
###    1. possible clean_up:
###            removing lone pairs
###            merging non_polar hydrogens
###            adding bonds to atoms with no bonds
###    2. making atoms in receptor conform to autodock3 atom types:
###            distinction between carbons and cyclic-aromatic carbons, 
###            polar hydrogens but no non-polar hydrogens
###    3. adding partial charges (Kollman by default)
###    4. writing 'pdbqs' file
###Input:  mols (MoleculeSet)
###Output: AD3receptor (Molecule)"""

###class Adt4PrepareReceptor(NetworkNode):
###    """ formats receptor for autodock4 using AutoDockTools.MoleculePreparation.AD4ReceptorPreparation.
###Process entails:
###    1. possible clean_up:
###            removing lone pairs
###            merging non_polar hydrogens
###            adding bonds to atoms with no bonds
###    2. making atoms in receptor conform to autodock4 atom types:
###            distinction between carbons and cyclic-aromatic carbons, 
###            distinction between hydrogen-bonding and non-hydrogen-bonding nitrogens,
###            no non-polar hydrogens
###    3. adding partial charges (gasteiger by default)
###    4. writing 'pdbqt' file
###Input:  mols (MoleculeSet)
###Output: AD4receptor (Molecule)"""
    

class AdtPrepareGpf3(NetworkNode):
    """ writes parameter file for autogrid3
Process entails:
    1.  setting ligand
    2.  setting receptor
    3.  setting specified values of various parameters
    4.  writing gpf file for autogrid3
Input:  ligand_filename, receptor_filename, optional parameter dictionary
Output: gpf ('string')"""
    
    def __init__(self, name='Prepare Autogrid3 Gpf', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )

        ip = self.inputPortsDescr
        ip.append(datatype='string', name='ligand_filename')
        ip.append(datatype='string', name='receptor_filename')
        ip.append(datatype='string', required=False, name='gpf_filename', defaultValue='')
        ip.append(datatype='dict', required=False, name='parameters', defaultValue={})
        ip.append(datatype='string', required=False, name='outputfilename', defaultValue='')

        op = self.outputPortsDescr
        op.append(datatype='string', name='ag3_parameter_file')

        code = """def doit(self, ligand_filename, receptor_filename, gpf_filename, parameters, outputfilename):
    if ligand_filename and receptor_filename:
        from AutoDockTools.GridParameters import GridParameterFileMaker
        gpfm = GridParameterFileMaker()
        gpfm.set_ligand(ligand_filename)
        gpfm.set_receptor(receptor_filename)
        if gpf_filename:
            gpfm.read_reference(gpf_filename)
        if len(parameters):
            gpfm.set_grid_parameters(parameters)
        if not outputfilename:
            outputfilename = gpfm.ligand.name+'_'+gpfm.receptor_stem + ".gpf"
        gpfm.write_gpf(outputfilename)
        self.outputData(ag3_parameter_file=outputfilename)\n"""

        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass



class AdtPrepareGpf4(NetworkNode):
    """ writes parameter file for autogrid4
Process entails:
    1.  setting ligand
    2.  setting receptor
    3.  setting specified values of various parameters
    4.  writing gpf file for autogrid4
Input:  ligand_filename, receptor_filename, optional parameter dictionary
Output: gpf ('string')"""
    
    def __init__(self, name='Prepare Autogrid4 Gpf', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )

        ip = self.inputPortsDescr
        ip.append(datatype='string', name='ligand_filename')
        ip.append(datatype='string', name='receptor_filename')
        ip.append(datatype='string', required=False, name='gpf_filename', defaultValue='')
        ip.append(datatype='dict', required=False, name='parameters', defaultValue={})
        ip.append(datatype='string', required=False, name='outputfilename', defaultValue='')

        op = self.outputPortsDescr
        op.append(datatype='string', name='ag4_parameter_file')

        code = """def doit(self, ligand_filename, receptor_filename, gpf_filename, parameters, outputfilename):
    if ligand_filename and receptor_filename:
        from AutoDockTools.GridParameters import GridParameter4FileMaker
        gpfm = GridParameter4FileMaker()
        gpfm.set_ligand(ligand_filename)
        gpfm.set_receptor(receptor_filename)
        if gpf_filename:
            gpfm.read_reference(gpf_filename)
        if len(parameters):
            gpfm.set_grid_parameters(parameters)
        if not outputfilename:
            outputfilename = gpfm.ligand.name+'_'+gpfm.receptor_stem + ".gpf"
        gpfm.write_gpf(outputfilename)
        self.outputData(ag4_parameter_file=outputfilename)\n"""

        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass



class AdtPrepareDpf3(NetworkNode):
    """ writes parameter file for autodock3
Process entails:
    1.  setting ligand
    2.  setting receptor
    3.  setting specified values of various parameters
    4.  writing dpf file for autogrid3
Input:  ligand_filename, receptor_filename, optional parameter dictionary
Output: dpf ('string')"""
    
    def __init__(self, name='Prepare Autodock3 Dpf', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )

        ip = self.inputPortsDescr
        ip.append(datatype='string', name='ligand_filename')
        ip.append(datatype='string', name='receptor_filename')
        ip.append(datatype='string', required=False, name='dpf_filename', defaultValue='')
        ip.append(datatype='dict', required=False, name='parameters', defaultValue={})
        ip.append(datatype='string', required=False, name='outputfilename', defaultValue='')

        op = self.outputPortsDescr
        op.append(datatype='string', name='ad3_parameter_file')

        code = """def doit(self, ligand_filename, receptor_filename, dpf_filename, parameters, outputfilename):
    if ligand_filename and receptor_filename:
        from AutoDockTools.DockingParameters import DockingParameterFileMaker
        dpfm = DockingParameterFileMaker()
        dpfm.set_ligand(ligand_filename)
        dpfm.set_receptor(receptor_filename)
        if dpf_filename:
            dpfm.read_reference(dpf_filename)
        if len(parameters):
            dpfm.set_docking_parameters(parameters)
        if not outputfilename:
            outputfilename = dpfm.ligand.name+'_'+dpfm.receptor_stem + ".dpf"
        dpfm.write_dpf(outputfilename)
        self.outputData(ad3_parameter_file=outputfilename)\n"""

        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass



class AdtPrepareDpf4(NetworkNode):
    """ writes parameter file for autodock4
Process entails:
    1.  setting ligand
    2.  setting receptor
    3.  setting specified values of various parameters
    4.  writing dpf file for autodock4
Input:  ligand_filename, receptor_filename, optional parameter dictionary
Output: dpf ('string')"""
    
    def __init__(self, name='Prepare Autodock4 Dpf', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )

        ip = self.inputPortsDescr
        ip.append(datatype='string', name='ligand_filename')
        ip.append(datatype='string', name='receptor_filename')
        ip.append(datatype='string', required=False, name='dpf_filename', defaultValue='')
        ip.append(datatype='dict', required=False, name='parameters', defaultValue={})
        ip.append(datatype='string', required=False, name='outputfilename', defaultValue='')

        op = self.outputPortsDescr
        op.append(datatype='string', name='ad4_parameter_file')

        code = """def doit(self, ligand_filename, receptor_filename, dpf_filename, parameters, outputfilename):
    if ligand_filename and receptor_filename:
        from AutoDockTools.DockingParameters import DockingParameter4FileMaker
        dpfm = DockingParameter4FileMaker()
        dpfm.set_ligand4(ligand_filename)
        dpfm.set_receptor4(receptor_filename)
        if dpf_filename:
            dpfm.read_reference4(dpf_filename)
        if len(parameters):
            dpfm.set_docking_parameters(parameters)
        if not outputfilename:
            outputfilename = dpfm.ligand.name+'_'+dpfm.receptor_stem + ".dpf"
        dpfm.write_dpf(outputfilename)
        self.outputData(ad4_parameter_file=outputfilename)\n"""

        self.setFunction(code)


    def myCallback(self, event=None):
        #self.paramPanel.run()
        pass


#class AdtPrepareFlexDocking4
#class AdtPdbqToPdbqt
#class AdtPdbqsToPdbqt
#class AdtGpf3ToGpf4
#class AdtDpf3ToDpf4
#class AdtRunAutogrid3
#class AdtRunAutogrid4
#class AdtRunAutodock3
#class AdtRunAutodock4
#class AdtSummarizeResults
#class AdtSummarizeResults4
#class AdtSummarizeXMLResults4
#class AdtPixelMapResults
#objects:
#MolecularPreparation classes
#   AutoDockBondClassifier
#   ReceptorWriter<==MolKitNodes/WriteMolecule
#   AD4ReceptorWriter<==MolKitNodes/WriteMolecule
#AD4FlexibleDockingPreparation
#prepare_ligand_dict
#ReceptorWriter
#AD4ReceptorWriter
#LigandPreparation
#AD4LigandPreparation
#AutoDock3_AtomTyper
#DockingParameters
#DockingParameterFileMaker
#GridParameters
#GridParameterFileMaker
#??Docking??


from Vision.VPE import NodeLibrary
adtlib = NodeLibrary('adtlib', '#4444FF')

#macros from other files in this directory
#from AutoDockTools.VisionInterface.PrepareAD4Molecule import PrepareAD4Molecule
#adtlib.addNode(PrepareAD4Molecule, 'Prepare AD4Molecule', 'Macro')
#from AutoDockTools.VisionInterface.AD4Ligands import AD4Ligands
#adtlib.addNode(AD4Ligands, 'Prepare AD4 Ligands', 'Macro')
#from AutoDockTools.VisionInterface.Prepare_AD4_Ligands import Prepare_AD4_Ligands
#adtlib.addNode(Prepare_AD4_Ligands, 'Prepare AD4 Ligands', 'Macro')
###from AutoDockTools.VisionInterface.PrepareAD3Ligand import PrepareAD3Ligand
###adtlib.addNode(PrepareAD3Ligand, 'Prepare AD3 Ligand', 'Macro')
###from AutoDockTools.VisionInterface.PrepareAD3Receptor import PrepareAD3Receptor
###adtlib.addNode(PrepareAD3Receptor, 'Prepare AD3Receptor', 'Macro')
###from AutoDockTools.VisionInterface.AD3Gpf import AD3Gpf
###adtlib.addNode(AD3Gpf, 'Prepare AD3 Gpf ', 'Macro')
###from AutoDockTools.VisionInterface.AD3Dpf import AD3Dpf
###adtlib.addNode(AD3Dpf, 'Prepare AD3 Dpf ', 'Macro')
#from AutoDockTools.VisionInterface.GPF4 import GPF4
#adtlib.addNode(GPF4, 'Prepare AD4 Gpf ', 'Macro')
from AutoDockTools.VisionInterface.Docking import Docking
adtlib.addNode(Docking, 'Docking', 'Macro')
from AutoDockTools.VisionInterface.recluster import recluster
adtlib.addNode(recluster, 'recluster...', 'Macro')

adtlib.addNode(GridParameterFileBrowserNE, 'Grid Parameter File Browser', 'Input')
adtlib.addNode(DockingParameterFileBrowserNE, 'Docking Parameter File Browser', 'Input')

#adtlib.addNode(ReadGridParameterFile, 'Read Grid Parameter File', 'Input')
#adtlib.addNode(ReadDockingParameterFile, 'Read Docking Parameter File', 'Input')


#adtlib.addNode(AdtPrepareGpf3, 'Prepare AD3Gpf', 'Macro')
#adtlib.addNode(AdtPrepareGpf4, 'Prepare AD4Gpf', 'Macro')
#adtlib.addNode(AdtPrepareDpf3, 'Prepare AD3Dpf', 'Macro')
#adtlib.addNode(AdtPrepareDpf4, 'Prepare AD4Dpf', 'Macro')

adtlib.addNode(Assign_AD4Types, 'AD4_typer', 'Mapper')
adtlib.addNode(Add_SolvationParameters, 'Add Solvation Parameters', 'Mapper')
adtlib.addNode(ManageRotatableBonds, 'Manage Rotatable Bonds', 'Mapper')
adtlib.addNode(MergeNonPolarHydrogens, 'Merge NonPolar Hydrogens', 'Mapper')
adtlib.addNode(MergeLonePairs, 'Merge Lone Pairs', 'Mapper')
adtlib.addNode(RemoveWaters, 'Remove Water Residues', 'Mapper')
adtlib.addNode(ManageAromaticCarbons, 'Manage Aromatic Carbons', 'Mapper')

adtlib.addNode(Ligand_Writer, 'Ligand Writer', 'Output')
adtlib.addNode(Ligand_Writer_AD4, 'AD4 Ligand Writer', 'Output')


try:
    UserLibBuild.addTypes(adtlib, 'MolKit.VisionInterface.MolKitTypes')
except Exception, e:
    print "loading types failed:", e




#
#
# A Gillet 08/2008
#

"""Unit test for statetocoords.py
"""

from AutoDockTools.Conformation import Conformation
from MolKit import Read
from mglutil.math.statetocoords import StateToCoords
import unittest, math
import numpy
from UserList import UserList

class StateToCoordsTest(unittest.TestCase):
    def setUp(self):
        self.decimals = 2 # for rounding; 7 is SciPy default.
        # value obtain from first run id 1 result of statecoords.xml
        self.origin =  numpy.array([ -1.413100, 0.115500, 0.685400])
        self.translation = numpy.array([1.144057, -0.003141, -7.605415])
        self.quaternion = numpy.array([ 0.284704, 0.255287, 0.535881, -0.752731])
        self.axisangle = numpy.array([0.432465, 0.387780, 0.814003, -82.345039])
        self.torsions = numpy.array([57.82, -175.79, 180.00, -162.36, 81.42, 100.35] )
        
        # value obtain from first run id 1 result of statecoords.dlg 
        self.expected_coords = numpy.array([(2.191,  -0.823,  -5.475),
                                            (1.363,  -0.143,  -9.679),
                                            (3.370,   1.839, -11.573)])
        
        self.lig = Read('statecoord_lig.pdbqt')[0]
        self.lig.buildBondsByDistance()    

    def test_statetocoords_from_axisangle(self):
        """ test getting molecule coordinate by using statetocoords with 
            axis/angle information 
        """
        self.lig.allAtoms.addConformation(self.lig.allAtoms.coords)  # preserve initial, work in new slot
        # we add a new conformation to ligand so we conserver original coords
        self.lig.allAtoms.addConformation(self.lig.allAtoms.coords)
        conf_index = len(self.lig.allAtoms[0]._coords) -1
        stoc = StateToCoords(self.lig, self.origin,conf_index)
        self.lig.stoc = stoc
        state= Conformation(self.lig, self.origin,self.translation,self.axisangle , self.torsions)
        self.lig.allAtoms.setConformation(conf_index)
        # get coordinate from state
        coords = state.getCoords()
        c0 = self.lig.allAtoms.coords[0]
        c16 = self.lig.allAtoms.coords[15]
        c22 = self.lig.allAtoms.coords[21]
        self.assertArrayEqual(self.expected_coords[0], c0)
        self.assertArrayEqual(self.expected_coords[1], c16)
        self.assertArrayEqual(self.expected_coords[2], c22)

    def test_statetocoords_from_quaternion(self):
        """ test getting molecule coordinate by using statetocoords with 
            quaternion information 
        """
        self.lig.allAtoms.addConformation(self.lig.allAtoms.coords)  # preserve initial, work in new slot
        # we add a new conformation to ligand so we conserver original coords
        self.lig.allAtoms.addConformation(self.lig.allAtoms.coords)
        conf_index = len(self.lig.allAtoms[0]._coords) -1
        stoc = StateToCoords(self.lig, self.origin,conf_index)
        self.lig.stoc = stoc
        quaternion= (self.quaternion[3],self.quaternion[:3])
        state= Conformation(self.lig, self.origin,self.translation,quaternion , self.torsions)
        self.lig.allAtoms.setConformation(conf_index)
        # get coordinate from state
        coords = state.getCoords()
        c0 = self.lig.allAtoms.coords[0]
        c16 = self.lig.allAtoms.coords[15]
        c22 = self.lig.allAtoms.coords[21]
        self.assertArrayEqual(self.expected_coords[0], c0)
        self.assertArrayEqual(self.expected_coords[1], c16)
        self.assertArrayEqual(self.expected_coords[2], c22)   
             
    def tearDown(self):
        pass       

    def assertArrayEqual(self, a1, a2, decimals=None):
        """Round the arrays according to decimals and compare

        Use self.decimals if decimals is not supplied"""
        if decimals:
            d = decimals
        else:
            d = self.decimals

        for point in xrange(len(a1)):
            self.assertAlmostEqual(a1[point],a2[point],d)

    
if __name__ == '__main__':
    unittest.main()   


## Automatically adapted for numpy.oldnumeric Jul 23, 2007 by 

#############################################################################
#
# Author: Ruth HUEY, Michel F. SANNER
#
# Copyright: M. Sanner TSRI 2000
#
#############################################################################


# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/energyCalculator.py,v 1.5 2007/07/24 17:30:44 vareille Exp $
#
# $Id: energyCalculator.py,v 1.5 2007/07/24 17:30:44 vareille Exp $
#
#

"""
This class calculates the AutoDock internal energy for a set of atoms.
"""

import numpy.oldnumeric as Numeric
from math import sqrt
from AutoDockTools.energyConstants import Rij, epsij
from MolKit.molecule import AtomSet


        
class EnergyCalculator:
    """This class knows how to calculate the energy of its set of atoms
    the Autodock way.  

    self.data: the list of atoms for the calculation
    self.reset: if 1, recalculates all the energies each time
    self.lowLim: number to be flag that calculation hasn't been done
    self.energy_type: internal, some day we may have others...
    
    """
    def __init__(self, data, reset=0, lowLim=-999999.0, energy_type='internal'):
        """data is a set of atoms
        """
        self.data = data
        self.data_ids = []
        for a in data:
            self.data_ids.append(id(a))
        #have to make sure that atom.number is in order
        self.data.sort()
        # save the pair-wise energies for reuse by get_energy
        # code: -1 if not set, 0 if within 1-4 distance, otherwise pairwise
        # energy
        self.energy_matrix = Numeric.zeros([len(data), len(data)]) + lowLim
        self.lowLim = lowLim
        self.setRotatable()
        self.setNeighbors()
        self.weedbonds()
        self.reset = reset
        self.NBCUTOFF = 8.0


    def setRotatable(self):
        #find the bonds which are in the torsion Map and label rotatable
        mols = self.data.top.uniq()
        for m in mols:
            if not hasattr(m, 'torTree'):
                continue
            l = []
            m.allAtoms.bonds[0].rotatable = 0
            for n in m.torTree.torsionMap:
                ats = m.allAtoms.get(lambda x, n=n: x.number-1 in n.bond)
                #ats = AtomSet([n.a,n.b])
                b = ats.bonds[0][0]
                b.rotatable = 1


    def weedbonds(self):
        mols = self.data.top.uniq()
        for m in mols:
            if not hasattr(m, 'torTree'):
                continue
            l = []
            atL = m.torTree.rootNode.atomList
            #print 'setting 0 for rootNode atomList:', atL
            for i in atL:
                for j in atL:
                    self.energy_matrix[i][j] = 0
            for n in m.torTree.torsionMap:
                atL = n.atomList
                atL.extend(n.bond)
                #print 'setting 0 for atomList:', atL
                for i in atL:
                    for j in atL:
                        self.energy_matrix[i][j] = 0


    def setNeighbors(self):
        """
        detect 1-2, 1-3 and 1-4 neighbors and set their entries in
        self.energy_matrix to 0
        FIX THIS: if bond is rotatable, don't reset the 1-4 pair
        """
        
        self.nb = {}
        for a1 in self.data:
            ind1 = self.data.index(a1)
            nbd =  {}
            nbd[ind1] = 0
            #nblist = self.nb[ind1] = [ind1]
            self.energy_matrix[ind1, ind1] = 0
            #set 1-2 interactions to 0
            for b in a1.bonds:
                a2 = b.atom1
                if id(a2)==id(a1): a2 = b.atom2
                if a2.number > a1.number:
                    #then do something here
                    ind2 = self.data.index(a2)
                    nbd[ind2] = 0
                    self.energy_matrix[ind1, ind2] = 0
                    self.energy_matrix[ind2, ind1] = 0
                #set 1-3 interactions to 0
                for b2 in a2.bonds:
                    a3 = b2.atom1
                    if id(a3)==id(a2): a3 = b2.atom2
                    if id(a3)==id(a1): continue
                    if a3.number > a1.number:
                        #then do something here
                        ind3 = self.data.index(a3)
                        nbd[ind3] = 0
                        self.energy_matrix[ind1, ind3] = 0
                        self.energy_matrix[ind3, ind1] = 0
                    #set 1-4 interactions to 0
                    #if rotatable, skip 1-4s for b2
                    if b2.rotatable:
                        #print 'skipping ', b2.atom1.name, '-', b2.atom2.name
                        continue
                    for b3 in a3.bonds:
                        #if hasattr(b3, 'activeTors') and b3.activeTors: continue
                        a4 = b3.atom1
                        if id(a4)==id(a3): a4 = b3.atom2
                        if id(a4)==id(a2): continue
                        if id(a4)==id(a1): continue
                        if a4.number > a1.number:
                            #then do something here
                            ind4 = self.data.index(a4)
                            nbd[ind4] = 0
                            self.energy_matrix[ind1, ind4] = 0
                            self.energy_matrix[ind4, ind1] = 0
            self.nb[ind1] = nbd.keys()


    def dist(self, a, b):
        """return distance between two atoms, a and b.
        """
        d = Numeric.array(b.coords) - Numeric.array(a.coords)
        return sqrt(Numeric.sum(d*d))

                        

    #def _get_energy_custom(self, a, b):
    #    """return energy between two atoms, a and b.
    #    """
    #    ax = self.data.index(a)
    #    bx = self.data.index(b)
#
#        if self.energy_matrix[ax][bx] >= 0.0:
#            # return previously saved energy
#            return self.energy_matrix[ax][bx]
#        else:
#            # compute, save, and return energy
#            #FIX THIS
#            dist = a.getRMSD_custom(b.getCoords())
#            self.energy_matrix[ax][bx] = self.energy_matrix[bx][ax] = dist
#            return dist


    def get_energy(self, a, b):
        """return energy between two atoms, a and b.
        """
        ax = self.data.index(a)
        bx = self.data.index(b)

        e = self.energy_matrix[ax][bx]
        if e > self.lowLim and not self.reset:
            # return previously saved energy
            return e
        elif e==0:
            # if getNeighbors set this to 0
            return e
        else:
            # compute, save, and return  energy
            energy = self.nbenergy(a,b)
            self.energy_matrix[ax][bx] = self.energy_matrix[bx][ax] = energy
            return energy


    def nbenergy(self, a, b):
        r = self.dist(a,b)
        if r>=self.NBCUTOFF:
            #print 'skipping ', a.number, '-', b.number
            return 0.0
        key = 'lj'+ a.element + b.element
        r_eq = Rij[key]
        ep = epsij[key]
        n = 12
        m = 6
        energy = (m/(n-m))*ep*r_eq**n/(r**n) - (n/(n-m))*ep*r_eq**m/r**m
        #energy = m*epsilon*r_eq**n/(r**n)/(n-m) - n*epsilon*r_eq**m/r**m/(n-m)
        return  energy


    def get_internal_energy(self):
        intEnergy = 0
        datalen = len(self.data)
        for i in range(datalen):
            a = self.data[i]
            for j in range(i+1, datalen):
                b = self.data[j]
                eij = self.get_energy(a,b)
                intEnergy = intEnergy + eij
        return intEnergy
                
    
    def get_subset_internal_energy(self, ats):
        subset_intEnergy = 0
        for a1 in ats:
            if id(a1) not in self.data_ids:
                print 'warning', a1.name, ' not in self.data'
            for a2 in ats:
                if id(a2) not in self.data_ids:
                    print 'warning', a2.name, ' not in self.data'
                eij = self.get_energy(a1, a2)
                subset_intEnergy = subset_intEnergy + eij
        return subset_intEnergy


    def reset_matrix(self):
        self.energy_matrix = Numeric.zeros([len(self.data), len(self.data)]) + self.lowLim
        self.weedbonds()
        self.setNeighbors()



    def print_nb_matrix(self):
        n = len(self.data)
        s = 'Atom: ID:'
        for i in range(1,n+1):
            s = s + '%2d'%i
        print s
        s = '_'*n
        print s
        em = self.energy_matrix
        for i in range(n):
            at = self.data[i]
            s = '  %2s     %2d'%(at.name,at.number)
            for j in range(n):
                if em[i][j]!=0:
                    s = s + '|x'
                else:
                    s = s + '|_'
            print s


    def print_half_nb_matrix(self):
        n = len(self.data)
        em = self.energy_matrix
        for i in range(n):
            s = '' + i*'_'
            for j in range(i, n):
                if em[i][j]!=0:
                    s = s + 'x'
                else:
                    s = s + '_'
            print s




########################################################################
#
#    Vision Macro - Python source code - file generated by vision
#    Wednesday 08 September 2010 12:32:33 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/VisionInterface/Adt/Macro/VirtualScreeningCSF.py,v 1.2 2010/09/08 19:38:57 jren Exp $
#
# $Id: VirtualScreeningCSF.py,v 1.2 2010/09/08 19:38:57 jren Exp $
#

from NetworkEditor.macros import MacroNode
from NetworkEditor.macros import MacroNode
class VirtualScreeningCSF(MacroNode):

    def __init__(self, constrkw={}, name='VirtualScreeningCSF', **kw):
        kw['name'] = name
        apply( MacroNode.__init__, (self,), kw)

    def beforeAddingToNetwork(self, net):
        MacroNode.beforeAddingToNetwork(self, net)
        from AutoDockTools.VisionInterface.Adt import Adt
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        net.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")
        from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
        try:
            addOpalServerAsCategory("http://ws.nbcr.net/opal2", replace=False)
        except:
            pass
        try:
            addOpalServerAsCategory("http://oolitevm.calit2.optiputer.net/opal2", replace=False)
        except:
            pass
        try:
            addOpalServerAsCategory("http://kryptonite.nbcr.net/opal2", replace=False)
        except:
            pass

    def afterAddingToNetwork(self):
        masterNet = self.macroNetwork
        from NetworkEditor.macros import MacroNode
        MacroNode.afterAddingToNetwork(self)
        from AutoDockTools.VisionInterface.Adt import Adt
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        ## building macro network ##
        VirtualScreeningCSF_120 = self
        from traceback import print_exc
        from AutoDockTools.VisionInterface.Adt import Adt
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        masterNet.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")
        from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
        try:
            addOpalServerAsCategory("http://ws.nbcr.net/opal2", replace=False)
        except:
            pass
        try:
            addOpalServerAsCategory("http://oolitevm.calit2.optiputer.net/opal2", replace=False)
        except:
            pass
        try:
            addOpalServerAsCategory("http://kryptonite.nbcr.net/opal2", replace=False)
        except:
            pass
        try:
            ## saving node input Ports ##
            input_Ports_121 = self.macroNetwork.ipNode
            apply(input_Ports_121.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore MacroInputNode named input Ports in network self.macroNetwork"
            print_exc()
            input_Ports_121=None

        try:
            ## saving node output Ports ##
            output_Ports_122 = self.macroNetwork.opNode
            apply(output_Ports_122.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            output_Ports_122.move(217, 330)
        except:
            print "WARNING: failed to restore MacroOutputNode named output Ports in network self.macroNetwork"
            print_exc()
            output_Ports_122=None

        try:
            ## saving node PrepareReceptor ##
            from Adt.Macro.PrepareReceptor import PrepareReceptor
            PrepareReceptor_141 = PrepareReceptor(constrkw={}, name='PrepareReceptor', library=Adt)
            self.macroNetwork.addNode(PrepareReceptor_141,115,96)
            apply(PrepareReceptor_141.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            Pdb2pqrWS_144 = PrepareReceptor_141.macroNetwork.nodes[2]
            Pdb2pqrOpalService_ws_nbcr_net_148 = Pdb2pqrWS_144.macroNetwork.nodes[3]
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['noopt'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['phi'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['psi'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['verbose'].widget.set(1, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['chain'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['nodebump'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['chi'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['ligand'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['hbond'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['with_ph'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['forcefield'].widget.set(r"AMBER", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['clean'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['inId'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['apbs_input'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['ffout'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['localRun'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['rama'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['execPath'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_148.inputPortByName['assign_only'].widget.set(0, run=False)
            GetURLFromList_149 = Pdb2pqrWS_144.macroNetwork.nodes[4]
            GetURLFromList_149.inputPortByName['ext'].widget.set(r"pqr", run=False)

            ## saving connections for network Pdb2pqrWS ##
            Pdb2pqrWS_144.macroNetwork.freeze()
            Pdb2pqrWS_144.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_145 = Pdb2pqrWS_144.macroNetwork.ipNode
            input_Ports_145.outputPorts[1].configure(name='CheckFileFormat_value')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_146 = Pdb2pqrWS_144.macroNetwork.opNode
            output_Ports_146.inputPorts[1].configure(singleConnection='auto')
            output_Ports_146.inputPorts[2].configure(singleConnection='auto')
            output_Ports_146.inputPorts[1].configure(name='UpdateReceptor_receptor_obj')
            output_Ports_146.inputPorts[2].configure(name='UpdateReceptor_pdb2pqr_result')
            Pdb2pqrWS_144.inputPorts[0].configure(name='CheckFileFormat_value')
            Pdb2pqrWS_144.inputPorts[0].configure(datatype='receptor')
            ## configure MacroNode input ports
            Pdb2pqrWS_144.outputPorts[0].configure(name='UpdateReceptor_receptor_obj')
            Pdb2pqrWS_144.outputPorts[0].configure(datatype='receptor')
            Pdb2pqrWS_144.outputPorts[1].configure(name='UpdateReceptor_pdb2pqr_result')
            Pdb2pqrWS_144.outputPorts[1].configure(datatype='string')
            ## configure MacroNode output ports
            Pdb2pqrWS_144.shrink()
            PrepareReceptorWS_151 = PrepareReceptor_141.macroNetwork.nodes[3]
            PrepareReceptorOpalService_ws_nbcr_net_155 = PrepareReceptorWS_151.macroNetwork.nodes[3]
            PrepareReceptorOpalService_ws_nbcr_net_155.inputPortByName['o'].widget.set(r"", run=False)
            PrepareReceptorOpalService_ws_nbcr_net_155.inputPortByName['v'].widget.set(0, run=False)
            PrepareReceptorOpalService_ws_nbcr_net_155.inputPortByName['localRun'].widget.set(0, run=False)
            PrepareReceptorOpalService_ws_nbcr_net_155.inputPortByName['execPath'].widget.set(r"", run=False)
            GetURLFromList_156 = PrepareReceptorWS_151.macroNetwork.nodes[4]
            GetURLFromList_156.inputPortByName['ext'].widget.set(r"pdbqt", run=False)
            DownloadToFile_157 = PrepareReceptorWS_151.macroNetwork.nodes[5]
            DownloadToFile_157.inputPortByName['overwrite'].widget.set(1, run=False)

            ## saving connections for network PrepareReceptorWS ##
            PrepareReceptorWS_151.macroNetwork.freeze()
            PrepareReceptorWS_151.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_152 = PrepareReceptorWS_151.macroNetwork.ipNode
            input_Ports_152.outputPorts[1].configure(name='CheckFileFormat_value')
            input_Ports_152.outputPorts[2].configure(name='PrepareReceptorOpalService_ws_nbcr_net_C')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_153 = PrepareReceptorWS_151.macroNetwork.opNode
            output_Ports_153.inputPorts[1].configure(singleConnection='auto')
            output_Ports_153.inputPorts[2].configure(singleConnection='auto')
            output_Ports_153.inputPorts[1].configure(name='UpdateReceptor_receptor_prepared_obj')
            output_Ports_153.inputPorts[2].configure(name='UpdateReceptor_receptor_result')
            PrepareReceptorWS_151.inputPorts[0].configure(name='CheckFileFormat_value')
            PrepareReceptorWS_151.inputPorts[0].configure(datatype='receptor')
            PrepareReceptorWS_151.inputPorts[1].configure(name='PrepareReceptorOpalService_ws_nbcr_net_C')
            PrepareReceptorWS_151.inputPorts[1].configure(datatype='boolean')
            ## configure MacroNode input ports
            PrepareReceptorWS_151.outputPorts[0].configure(name='UpdateReceptor_receptor_prepared_obj')
            PrepareReceptorWS_151.outputPorts[0].configure(datatype='receptor_prepared')
            PrepareReceptorWS_151.outputPorts[1].configure(name='UpdateReceptor_receptor_result')
            PrepareReceptorWS_151.outputPorts[1].configure(datatype='string')
            ## configure MacroNode output ports
            PrepareReceptorWS_151.shrink()

            ## saving connections for network PrepareReceptor ##
            PrepareReceptor_141.macroNetwork.freeze()
            PrepareReceptor_141.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_142 = PrepareReceptor_141.macroNetwork.ipNode
            input_Ports_142.outputPorts[1].configure(name='Pdb2pqrWS_CheckFileFormat_value')
            input_Ports_142.outputPorts[2].configure(name='PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_143 = PrepareReceptor_141.macroNetwork.opNode
            output_Ports_143.inputPorts[1].configure(singleConnection='auto')
            output_Ports_143.inputPorts[2].configure(singleConnection='auto')
            output_Ports_143.inputPorts[1].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_prepared_obj')
            output_Ports_143.inputPorts[2].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_result')
            PrepareReceptor_141.inputPorts[0].configure(name='Pdb2pqrWS_CheckFileFormat_value')
            PrepareReceptor_141.inputPorts[0].configure(datatype='receptor')
            PrepareReceptor_141.inputPorts[1].configure(name='PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')
            PrepareReceptor_141.inputPorts[1].configure(datatype='boolean')
            ## configure MacroNode input ports
            PrepareReceptor_141.outputPorts[0].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_prepared_obj')
            PrepareReceptor_141.outputPorts[0].configure(datatype='receptor_prepared')
            PrepareReceptor_141.outputPorts[1].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_result')
            PrepareReceptor_141.outputPorts[1].configure(datatype='string')
            ## configure MacroNode output ports
            PrepareReceptor_141.shrink()
            apply(PrepareReceptor_141.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore PrepareReceptor named PrepareReceptor in network self.macroNetwork"
            print_exc()
            PrepareReceptor_141=None

        try:
            ## saving node ComputeGrids ##
            from Adt.Macro.ComputeGrids import ComputeGrids
            ComputeGrids_159 = ComputeGrids(constrkw={}, name='ComputeGrids', library=Adt)
            self.macroNetwork.addNode(ComputeGrids_159,136,207)
            apply(ComputeGrids_159.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            prepareGPF_kryptonite_nbcr_net_163 = ComputeGrids_159.macroNetwork.nodes[3]
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['singlelib'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['r_url'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['zpoints'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['filter_file_url'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['lib'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['ypoints'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['xcenter'].widget.set(r"auto", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['p'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['o'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['zcenter'].widget.set(r"auto", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['v'].widget.set(0, run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['userlib'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['xpoints'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['localRun'].widget.set(0, run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['ycenter'].widget.set(r"auto", run=False)
            prepareGPF_kryptonite_nbcr_net_163.inputPortByName['execPath'].widget.set(r"", run=False)
            autogrid_kryptonite_nbcr_net_164 = ComputeGrids_159.macroNetwork.nodes[4]
            autogrid_kryptonite_nbcr_net_164.inputPortByName['infile_url'].widget.set(r"", run=False)
            autogrid_kryptonite_nbcr_net_164.inputPortByName['l'].widget.set(r"output.glg", run=False)
            autogrid_kryptonite_nbcr_net_164.inputPortByName['o'].widget.set(0, run=False)
            autogrid_kryptonite_nbcr_net_164.inputPortByName['p'].widget.set(r"", run=False)
            autogrid_kryptonite_nbcr_net_164.inputPortByName['localRun'].widget.set(0, run=False)
            autogrid_kryptonite_nbcr_net_164.inputPortByName['execPath'].widget.set(r"", run=False)
            GetURLFromList_167 = ComputeGrids_159.macroNetwork.nodes[7]
            GetURLFromList_167.inputPortByName['ext'].widget.set(r"gpf", run=False)

            ## saving connections for network ComputeGrids ##
            ComputeGrids_159.macroNetwork.freeze()
            ComputeGrids_159.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_160 = ComputeGrids_159.macroNetwork.ipNode
            input_Ports_160.outputPorts[1].configure(name='GetComputeGridsInputs_ligands')
            input_Ports_160.outputPorts[2].configure(name='GetComputeGridsInputs_receptor_pdbqt')
            input_Ports_160.outputPorts[3].configure(name='GetComputeGridsInputs_gpf_obj')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_161 = ComputeGrids_159.macroNetwork.opNode
            output_Ports_161.inputPorts[1].configure(singleConnection='auto')
            output_Ports_161.inputPorts[2].configure(singleConnection='auto')
            output_Ports_161.inputPorts[1].configure(name='MakeAutogridResultObj_autogrid_result_obj')
            output_Ports_161.inputPorts[2].configure(name='GetMainURLFromList_newurl')
            ComputeGrids_159.inputPorts[0].configure(name='GetComputeGridsInputs_ligands')
            ComputeGrids_159.inputPorts[0].configure(datatype='LigandDB')
            ComputeGrids_159.inputPorts[1].configure(name='GetComputeGridsInputs_receptor_pdbqt')
            ComputeGrids_159.inputPorts[1].configure(datatype='receptor_prepared')
            ComputeGrids_159.inputPorts[2].configure(name='GetComputeGridsInputs_gpf_obj')
            ComputeGrids_159.inputPorts[2].configure(datatype='gpf_template')
            ## configure MacroNode input ports
            ComputeGrids_159.outputPorts[0].configure(name='MakeAutogridResultObj_autogrid_result_obj')
            ComputeGrids_159.outputPorts[0].configure(datatype='autogrid_results')
            ComputeGrids_159.outputPorts[1].configure(name='GetMainURLFromList_newurl')
            ComputeGrids_159.outputPorts[1].configure(datatype='string')
            ## configure MacroNode output ports
            ComputeGrids_159.shrink()
            apply(ComputeGrids_159.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore ComputeGrids named ComputeGrids in network self.macroNetwork"
            print_exc()
            ComputeGrids_159=None

        try:
            ## saving node AutodockVsCSF ##
            from Adt.Macro.AutodockVsCSF import AutodockVsCSF
            AutodockVsCSF_168 = AutodockVsCSF(constrkw={}, name='AutodockVsCSF', library=Adt)
            self.macroNetwork.addNode(AutodockVsCSF_168,234,272)
            apply(AutodockVsCSF_168.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            autodock_kryptonite_nbcr_net_172 = AutodockVsCSF_168.macroNetwork.nodes[3]
            autodock_kryptonite_nbcr_net_172.inputPortByName['ga_run'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_172.inputPortByName['lib'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_172.inputPortByName['filter_file_url'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_172.inputPortByName['ga_num_evals'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_172.inputPortByName['sched'].widget.set(r"CSF", run=False)
            autodock_kryptonite_nbcr_net_172.inputPortByName['ga_num_generations'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_172.inputPortByName['userlib'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_172.inputPortByName['ga_pop_size'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_172.inputPortByName['localRun'].widget.set(0, run=False)
            autodock_kryptonite_nbcr_net_172.inputPortByName['email'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_172.inputPortByName['execPath'].widget.set(r"", run=False)
            AutodockSlice_oolitevm_calit2_optiputer_net_175 = AutodockVsCSF_168.macroNetwork.nodes[6]
            AutodockSlice_oolitevm_calit2_optiputer_net_175.inputPortByName['execPath'].widget.set(r"", run=False)
            AutodockSlice_oolitevm_calit2_optiputer_net_175.inputPortByName['localRun'].widget.set(0, run=False)
            AutodockSlice_oolitevm_calit2_optiputer_net_175.inputPortByName['divisor'].widget.set(r"", run=False)

            ## saving connections for network AutodockVsCSF ##
            AutodockVsCSF_168.macroNetwork.freeze()
            AutodockVsCSF_168.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_169 = AutodockVsCSF_168.macroNetwork.ipNode
            input_Ports_169.outputPorts[1].configure(name='PrepareADVSInputs_ligands')
            input_Ports_169.outputPorts[2].configure(name='PrepareADVSInputs_autogrid_results')
            input_Ports_169.outputPorts[3].configure(name='PrepareADVSInputs_dpf_template_obj')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_170 = AutodockVsCSF_168.macroNetwork.opNode
            output_Ports_170.inputPorts[1].configure(singleConnection='auto')
            output_Ports_170.inputPorts[1].configure(name='GetMainURLFromList_newurl')
            AutodockVsCSF_168.inputPorts[0].configure(name='PrepareADVSInputs_ligands')
            AutodockVsCSF_168.inputPorts[0].configure(datatype='LigandDB')
            AutodockVsCSF_168.inputPorts[1].configure(name='PrepareADVSInputs_autogrid_results')
            AutodockVsCSF_168.inputPorts[1].configure(datatype='autogrid_results')
            AutodockVsCSF_168.inputPorts[2].configure(name='PrepareADVSInputs_dpf_template_obj')
            AutodockVsCSF_168.inputPorts[2].configure(datatype='dpf_template')
            ## configure MacroNode input ports
            AutodockVsCSF_168.outputPorts[0].configure(name='GetMainURLFromList_newurl')
            AutodockVsCSF_168.outputPorts[0].configure(datatype='string')
            ## configure MacroNode output ports
            AutodockVsCSF_168.shrink()
            apply(AutodockVsCSF_168.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore AutodockVsCSF named AutodockVsCSF in network self.macroNetwork"
            print_exc()
            AutodockVsCSF_168=None

        #self.macroNetwork.run()
        self.macroNetwork.freeze()

        ## saving connections for network VirtualScreeningCSF ##
        input_Ports_121 = self.macroNetwork.ipNode
        if input_Ports_121 is not None and PrepareReceptor_141 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_121, PrepareReceptor_141, "new", "Pdb2pqrWS_CheckFileFormat_value", blocking=True
                    , splitratio=[0.70852617526621731, 0.39749336046566702])
            except:
                print "WARNING: failed to restore connection between input_Ports_121 and PrepareReceptor_141 in network self.macroNetwork"
        if PrepareReceptor_141 is not None and ComputeGrids_159 is not None:
            try:
                self.macroNetwork.connectNodes(
                    PrepareReceptor_141, ComputeGrids_159, "PrepareReceptorWS_UpdateReceptor_receptor_prepared_obj", "GetComputeGridsInputs_receptor_pdbqt", blocking=True
                    , splitratio=[0.35807693100671723, 0.50862337251735745])
            except:
                print "WARNING: failed to restore connection between PrepareReceptor_141 and ComputeGrids_159 in network self.macroNetwork"
        if input_Ports_121 is not None and ComputeGrids_159 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_121, ComputeGrids_159, "new", "GetComputeGridsInputs_ligands", blocking=True
                    , splitratio=[0.40102167589221782, 0.6253111040879471])
            except:
                print "WARNING: failed to restore connection between input_Ports_121 and ComputeGrids_159 in network self.macroNetwork"
        if input_Ports_121 is not None and AutodockVsCSF_168 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_121, AutodockVsCSF_168, "ComputeGrids_GetComputeGridsInputs_ligands", "PrepareADVSInputs_ligands", blocking=True
                    , splitratio=[0.65260935115862417, 0.64296744661832661])
            except:
                print "WARNING: failed to restore connection between input_Ports_121 and AutodockVsCSF_168 in network self.macroNetwork"
        if ComputeGrids_159 is not None and AutodockVsCSF_168 is not None:
            try:
                self.macroNetwork.connectNodes(
                    ComputeGrids_159, AutodockVsCSF_168, "MakeAutogridResultObj_autogrid_result_obj", "PrepareADVSInputs_autogrid_results", blocking=True
                    , splitratio=[0.50054097826587762, 0.55954672420260165])
            except:
                print "WARNING: failed to restore connection between ComputeGrids_159 and AutodockVsCSF_168 in network self.macroNetwork"
        if input_Ports_121 is not None and ComputeGrids_159 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_121, ComputeGrids_159, "new", "GetComputeGridsInputs_gpf_obj", blocking=True
                    , splitratio=[0.73607423731241384, 0.54225043834950515])
            except:
                print "WARNING: failed to restore connection between input_Ports_121 and ComputeGrids_159 in network self.macroNetwork"
        if input_Ports_121 is not None and AutodockVsCSF_168 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_121, AutodockVsCSF_168, "new", "PrepareADVSInputs_dpf_template_obj", blocking=True
                    , splitratio=[0.24404719659126095, 0.41822006352735946])
            except:
                print "WARNING: failed to restore connection between input_Ports_121 and AutodockVsCSF_168 in network self.macroNetwork"
        if input_Ports_121 is not None and PrepareReceptor_141 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_121, PrepareReceptor_141, "new", "PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C", blocking=True
                    , splitratio=[0.73098254809437191, 0.23999050334763958])
            except:
                print "WARNING: failed to restore connection between input_Ports_121 and PrepareReceptor_141 in network self.macroNetwork"
        output_Ports_122 = self.macroNetwork.opNode
        if AutodockVsCSF_168 is not None and output_Ports_122 is not None:
            try:
                self.macroNetwork.connectNodes(
                    AutodockVsCSF_168, output_Ports_122, "GetMainURLFromList_newurl", "new", blocking=True
                    , splitratio=[0.55528494083678659, 0.33476142488031579])
            except:
                print "WARNING: failed to restore connection between AutodockVsCSF_168 and output_Ports_122 in network self.macroNetwork"
        self.macroNetwork.runOnNewData.value = False

        ## modifying MacroInputNode dynamic ports
        input_Ports_121 = self.macroNetwork.ipNode
        input_Ports_121.outputPorts[1].configure(name='PrepareReceptor_Pdb2pqrWS_CheckFileFormat_value')
        input_Ports_121.outputPorts[2].configure(name='ComputeGrids_GetComputeGridsInputs_ligands')
        input_Ports_121.outputPorts[3].configure(name='ComputeGrids_GetComputeGridsInputs_gpf_obj')
        input_Ports_121.outputPorts[4].configure(name='AutodockVsCSF_PrepareADVSInputs_dpf_template_obj')
        input_Ports_121.outputPorts[5].configure(name='PrepareReceptor_PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')

        ## modifying MacroOutputNode dynamic ports
        output_Ports_122 = self.macroNetwork.opNode
        output_Ports_122.inputPorts[1].configure(singleConnection='auto')
        output_Ports_122.inputPorts[1].configure(name='AutodockVsCSF_GetMainURLFromList_newurl')
        ## configure MacroNode input ports
        VirtualScreeningCSF_120.inputPorts[0].configure(name='PrepareReceptor_Pdb2pqrWS_CheckFileFormat_value')
        VirtualScreeningCSF_120.inputPorts[0].configure(datatype='receptor')
        VirtualScreeningCSF_120.inputPorts[1].configure(name='ComputeGrids_GetComputeGridsInputs_ligands')
        VirtualScreeningCSF_120.inputPorts[1].configure(datatype='LigandDB')
        VirtualScreeningCSF_120.inputPorts[2].configure(name='ComputeGrids_GetComputeGridsInputs_gpf_obj')
        VirtualScreeningCSF_120.inputPorts[2].configure(datatype='gpf_template')
        VirtualScreeningCSF_120.inputPorts[3].configure(name='AutodockVsCSF_PrepareADVSInputs_dpf_template_obj')
        VirtualScreeningCSF_120.inputPorts[3].configure(datatype='dpf_template')
        VirtualScreeningCSF_120.inputPorts[4].configure(name='PrepareReceptor_PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')
        VirtualScreeningCSF_120.inputPorts[4].configure(datatype='boolean')
        ## configure MacroNode output ports
        VirtualScreeningCSF_120.outputPorts[0].configure(name='AutodockVsCSF_GetMainURLFromList_newurl')
        VirtualScreeningCSF_120.outputPorts[0].configure(datatype='string')

        VirtualScreeningCSF_120.shrink()

        ## reset modifications ##
        VirtualScreeningCSF_120.resetTags()
        VirtualScreeningCSF_120.buildOriginalList()



#############################################################################
# 
# Graph class: 
# Written by  John E. Grayson 
#
# InteractiveHistogramBuilder class written by Ruth Huey
#
#
#############################################################################


import sys
import Tkinter
from Canvas import Line, CanvasText, Rectangle
import tkFileDialog
import string, math, os

from mglutil.util.misc import ensureFontCase
from mglutil.gui import widgetsOnBackWindowsCanGrabFocus

def minBound(nodeList):
    x = 10000000
    y = 10000000
    for x1, y1 in  nodeList:
        if x1 < x: x= x1
        if y1 < y: y= y1
    return x, y


def maxBound(nodeList):
    x = -10000000
    y = -10000000
    for x1, y1 in  nodeList:
        if x1 > x: x= x1
        if y1 > y: y= y1
    return x, y




class InteractiveGraphBuilder(Tkinter.Frame):


    def __init__(self, name='NoName', master=None, nodeList=[], **kw):
        """
        """
        self.name = name
        self.nodes = {}
        self.geoms = {}

        if kw.has_key('visibleWidth'):
            self.visibleWidth = visibleWidth
            del kw['visibleWidth']
        else:
            self.visibleWidth = 400

        if kw.has_key('visibleHeight'):
            self.visibleWidth = visibleHeight
            del kw['visibleHeight']
        else:
            self.visibleHeight = 400

        if kw.has_key('totalWidth'):
            self.visibleWidth = totalWidth
            del kw['totalWidth']
        else:
            self.totalWidth = 400

        if kw.has_key('totalHeight'):
            self.totalHeight = totalHeight
            del kw['totalHeight']
        else:
            self.totalHeight = 400

        if kw.has_key('hasScroll'):
            self.hasScroll = hasScroll
            del kw['hasScroll']
        else:
            self.hasScroll = 0

        if kw.has_key('font'):
            self.font = font
            del kw['font']
        else:
            self.font = (ensureFontCase('helvetica'), 14)

        Tkinter.Frame.__init__(self, master)
        Tkinter.Pack.config(self, expand=1, fill=Tkinter.BOTH)
        apply(self.createCanvas, (), kw)
        self.paramPanelObject = None
        self.plotarea_size = [None, None]
        Tkinter.Widget.bind(self.draw, '<Enter>', self.enter)
        # bind mouse button callbacks
        self.draw.bind("<Any-ButtonPress-1>", self.mouse1DownCanvas)
        self.draw.bind("<Any-ButtonPress-3>", self.mouse1DownCanvas)
        self.draw.bind("<Any-ButtonPress-2>", self.mouse1DownCanvas)
        self.draw.bind("<Any-Double-ButtonPress-1>", self.mouse2DownCanvas)
        self.draw.bind("<Any-Double-ButtonPress-2>", self.mouse2DownCanvas)
        self.draw.bind("<Any-Double-ButtonPress-3>", self.mouse2DownCanvas)
        self.draw.old_current = None

    
    def mouse1DownCanvas(self, event=None):
        #print 'in mouse1DC'
        obj = self.draw.find_withtag('current')
        if self.draw.old_current:
            self.draw.itemconfig(self.draw.old_current, fill = 'blue')
        self.draw.itemconfig('current', fill = 'red')
        self.draw.old_current = obj
        #print obj
        if len(obj) and obj[0] in self.geoms.keys():
            self.currentNode = self.geoms[obj[0]]
            #print 'currentNode=', self.currentNode
            if hasattr(self, 'reverseIndex'):
                #print self.reverseIndex[self.currentNode.info]
                self.currentInfo = self.reverseIndex[self.currentNode.info]


    def mouse2DownCanvas(self, event=None):
        #print 'in mouse2DownCanvas'
        pass



    def enter(self, event=None):
        try:
            if widgetsOnBackWindowsCanGrabFocus is False:
                lActiveWindow = self.draw.focus_get()
                if    lActiveWindow is not None \
                  and \
                    ( lActiveWindow.winfo_toplevel() != self.draw.winfo_toplevel() ):
                    return
        except:
            pass
        self.draw.focus_set()

    
    def createCanvas(self):
        """ create the Canvas and Title widgets"""
        self.widgetArea = Tkinter.Frame(self, borderwidth=2, relief='sunken')
        self.createMenus()
        self.canvasFrame = Tkinter.Frame(self)
        #if self.hasScroll:
        self.scrollregion = [0, 0, self.totalWidth, self.totalHeight]
        self.visibleOriginX = 0
        self.visibleOriginY = 0
        self.draw = Tkinter.Canvas(self.canvasFrame, width=self.visibleWidth,
                        height=self.visibleHeight,
                        background='white',
                        scrollregion=tuple(self.scrollregion))
        self.draw.scrollX = Tkinter.Scrollbar(self.canvasFrame,
                                        orient=Tkinter.HORIZONTAL)
        self.draw.scrollY = Tkinter.Scrollbar(self.canvasFrame,
                                        orient=Tkinter.VERTICAL)
        self.draw.bind('<Configure>', self.configure)

        self.draw['xscrollcommand'] = self.draw.scrollX.set
        self.draw['yscrollcommand'] = self.draw.scrollY.set
        self.draw.scrollX['command'] = self.myxview
        self.draw.scrollY['command'] = self.myyview
        self.canvasFrame.pack(side=Tkinter.LEFT,expand=1,fill=Tkinter.BOTH)
        self.draw.scrollX.pack(side=Tkinter.BOTTOM,fill=Tkinter.X)
        self.draw.scrollY.pack(side=Tkinter.LEFT,expand=1,fill=Tkinter.BOTH)
        self.draw.pack(side=Tkinter.LEFT,expand=1,fill=Tkinter.BOTH)
        border_w = self.draw.winfo_reqwidth() - self.visibleWidth
        border_h = self.draw.winfo_reqheight() - self.visibleHeight
        self.border = (border_w, border_h)


    def myxview(self, command, value):
        self.visibleOriginX = int(self.totalWidth*float(value))
        self.draw.xview(command, value)


    def myyview(self, command, value):
        self.visibleOriginY = int(self.totalHeight*float(value))
        self.draw.yview(command, value)


    def configure(self, event=None):
        new_width = event.width - self.border[0]
        new_height = event.height - self.border[1]
        width = int(self.draw.cget('width'))
        height = int(self.draw.cget('height'))
        if new_width==width and new_height==height:
            return
        self.draw.configure(width=new_width, height=new_height)
        self._setsize()
        self.visibleWidth = new_width
        self.visibleHeight = new_height
        self.clear()
        self.replot()

    

    def _setsize(self):
        self.width = int(self.draw.cget('width'))
        self.height = int(self.draw.cget('height'))
        #self.plotarea_size[0] = 0.97 * self.width
        #self.plotarea_size[1] = 0.97 * -self.height      
        #xo = 0.5*(self.width-self.plotarea_size[0])
        #yo = 0.5*(self.height-self.plotarea_size[1])
        #self.plotarea_origin = (xo, yo)
        self.plotarea_size = [0.96 * self.width, -0.90 * self.height]
        self.plotarea_origin = (0.030*self.width, 0.96*self.height)
        #self.plotarea_origin = (0.016*self.width, 0.96*self.height)


    def createMenus(self):
        self.mBar = Tkinter.Frame(self, relief=Tkinter.RAISED,borderwidth=2)
        self.mBar.pack(fill=Tkinter.X)
        self.menuButtons = {}
        self.makeFileMenu()
        self.makeEditMenu()
        apply(self.mBar.tk_menuBar, self.menuButtons.values())
        self.title = Tkinter.Label(self.mBar, text=self.name)
        self.title.pack(side=Tkinter.RIGHT)


    def makeFileMenu(self):
        File_button = Tkinter.Menubutton(self.mBar, text='File',underline=0)
        self.menuButtons['File'] = File_button
        File_button.pack(side = Tkinter.LEFT, padx='1m')
        File_button.menu = Tkinter.Menu(File_button)
        #File_button.menu.add_command(label='Load...', underline=0,
                                    #command = self.loadFile)
        File_button.menu.add_command(label='Exit...', underline=0,
                                    command = self.exit)
        File_button['menu'] = File_button.menu


    def loadFile(self, event=None):
        pass


    def exit(self, event=None):
        self.master.destroy()


    def makeEditMenu(self, event=None):
        Edit_button = Tkinter.Menubutton(self.mBar, text='Edit',underline=0)
        self.menuButtons['Edit'] = Edit_button
        Edit_button.pack(side = Tkinter.LEFT, padx='1m')
        Edit_button.menu = Tkinter.Menu(Edit_button)
        Edit_button.menu.add_command(label='Clear...', underline=0,
                                    command = self.clear)
        Edit_button.menu.add_command(label='Redraw...', underline=0,
                                    command = self.replot)
        #Edit_button.menu.add_command(label='Delete...', underline=0,
                                    #command = self.delete)
        Edit_button.menu.add_command(label='Write...', underline=0,
                                    command = self.printCanvas)
        Edit_button['menu'] = Edit_button.menu


    def clear(self, event=None):
        self.geoms = {}
        self.draw.delete('all')


    def delete(self, event=None):
        pass


    def replot(self):
        if self.last_drawn is not None:
            apply(self.buildIt, (self.last_drawn,), {})


    def printCanvas(self, idir=None, ifile=None, title=None):
        #other options are 'colormap', 'colormode', 'fontmap', 'height'
        #'pageanchor', 'pageheight','pagewidth', 'pagex','pagey','rotate'
        #'width', 'x', 'y'
        types = [('Postscript Files', '*.ps')]
        if idir==None:
            idir = os.curdir
        ifile = ifile
        if title==None:
            title='Postscript File'
        file = tkFileDialog.asksaveasfilename( filetypes=types,
                                           initialdir=idir,
                                           initialfile=ifile,
                                           title=title)
        self.draw.postscript({'file':file,'colormode':'color'})


    def buildIt(self, graphics, xaxis='automatic', yaxis='custom',
            xlabels='float', ylabels='int'):
        #NB: p1 is top left point of bounding box
        p1, p2 = graphics.boundingBox()
        xaxis = self._axisInterval(xaxis, p1[0], p2[0])
        yaxis = self._axisInterval(yaxis, p1[1], p2[1])
        text_width = [0., 0.]
        text_height = [0., 0.]
        if xaxis is not None:
            p1 = xaxis[0], p1[1]
            p2 = xaxis[1], p2[1]
            xticks = self._ticks(xaxis[0], xaxis[1], form=xlabels)
            bb = self._textBoundingBox(xticks[0][1])
            text_height[1] = bb[3]-bb[1]
            text_width[0] = 0.5*(bb[2]-bb[0])
            bb = self._textBoundingBox(xticks[-1][1])
            text_width[1] = 0.5*(bb[2]-bb[0])
        else:
            xticks = None
        if yaxis is not None:
            p1 = p1[0], yaxis[0]
            p2 = p2[0], yaxis[1]
            yticks = self._ticks(yaxis[0], yaxis[1], form=ylabels)
            for y in yticks:
                bb = self._textBoundingBox(y[1])
                w = bb[2]-bb[0]
                text_width[0] = max(text_width[0], w)
            h = 0.5*(bb[3]-bb[1])
            text_height[0] = h
            text_height[1] = max(text_height[1], h)
        else:
            yticks = None
        text1 = [text_width[0], -text_height[1]]
        text2 = [text_width[1], -text_height[0]]
        self.p1 = p1
        self.p2 = p2
        scale = ((self.plotarea_size[0]-text1[0]-text2[0]) / \
                 (p2[0]-p1[0]),
                 (self.plotarea_size[1]-text1[1]-text2[1]) / \
                 (p2[1]-p1[1]))
        shift = ((-p1[0]*scale[0]) + self.plotarea_origin[0] + \
                 text1[0],
                 (-p1[1]*scale[1]) + self.plotarea_origin[1] + \
                 text1[1])
        self._drawAxes(self.draw, xaxis, yaxis, p1, p2,
                        scale, shift, xticks, yticks)
        graphics.fitToScale(scale, shift, p1, p2)
        graphics.buildIcons(self) 
        if self.label_text:
            pt1 = self.plotarea_origin[0] + self.width/2
            pt2 = (self.height + self.plotarea_size[1])/2.0
            self.draw.create_text(pt1,pt2,text=self.label_text,anchor='s',font=self.font)
        if self.xlabel_text:
            pt1 = self.plotarea_origin[0] + self.width/2
            pt2 = self.plotarea_origin[1] - self.spacing
            self.draw.create_text(pt1,pt2,text=self.xlabel_text,anchor='n',font=self.font)
        if self.ylabel_text:
            pt1 = (self.plotarea_origin[0] - self.visibleOriginX)/3.0
            #pt1 = self.plotarea_origin[0]
            #pt1 = self.plotarea_origin[0] + self.spacing/8
            pt2 = self.plotarea_origin[1]/2
            self.draw.create_text(pt1,pt2,text=self.ylabel_text,anchor='w',font=self.font)
        self.last_drawn = graphics


    def _axisInterval(self, spec, lower, upper):
        if spec==None: return None
        if spec=='minimal':
            if lower==upper:
                return lower-0.5, upper+0.5
            else:
                return lower, upper
        if spec=='automatic' or 'custom':
            range = upper-lower
            if range==0:
                return lower-0.5, upper+0.5
            log = math.log10(range)
            power = math.floor(log)
            fraction = log - power
            if fraction<=0.05:
                power = power-1
            grid = 10.**power
            lower = lower - lower% grid
            mod = upper % grid
            if mod!=0:
                upper = upper - mod + grid
            if spec == 'custom':
                lower = 0.0
            return lower, upper
        if type(spec)==type(()):
            lower, upper = spec
            if lower <= upper:
                return lower, upper
            else: return upper, lower
        raise ValueError, str(spec)+': illegal axis specification'


    def _drawAxes(self, canvas, xaxis, yaxis,
                  bb1, bb2, scale, shift, xticks, yticks):
        dict = {'anchor': Tkinter.N, 'fill': 'black'}
        if self.font is not None:
            dict['font'] = self.font
        if xaxis is not None:
            lower, upper = xaxis
            text = 1
            for y, d in [(bb1[1], -3), (bb2[1], 3)]:
                p1 = (scale[0]*lower)+shift[0], (scale[1]*y)+shift[1]
                p2 = (scale[0]*upper)+shift[0], (scale[1]*y)+shift[1]
                Line(self.draw, p1[0], p1[1], p2[0], p2[1],
                     fill = 'black', width = 1)
                if xticks:
                    for x, label in xticks:
                        p = (scale[0]*x)+shift[0], \
                            (scale[1]*y)+shift[1]
                        Line(self.draw, p[0], p[1], p[0], p[1]+d,
                             fill = 'black', width = 1)
                        if text:
                            dict['text'] = label
                            apply(CanvasText, (self.draw, p[0],
                                               p[1]), dict)
                text = 0
        dict['anchor'] = Tkinter.E
        if yaxis is not None:
            lower, upper = yaxis
            text = 1
            for x, d in [(bb1[0], -3), (bb2[0], 3)]:
                p1 = (scale[0]*x)+shift[0], (scale[1]*lower)+shift[1]
                p2 = (scale[0]*x)+shift[0], (scale[1]*upper)+shift[1]
                Line(self.draw, p1[0], p1[1], p2[0], p2[1],
                     fill = 'black', width = 1)
                if yticks:
                    for y, label in yticks:
                        p = (scale[0]*x)+shift[0], \
                            (scale[1]*y)+shift[1]
                        Line(self.draw, p[0], p[1], p[0]-d, p[1],
                             fill = 'black', width = 1)
                        if text:
                            dict['text'] = label
                            apply(CanvasText,(self.draw,
                                              p[0]-2,p[1]), dict)
                text = 0


    def _ticks(self, lower, upper, form='float'):
        ideal = (upper-lower)/7.
        log = math.log10(ideal)
        power = math.floor(log)
        fraction = log-power
        factor = 1.
        error = fraction
        for f, lf in self._multiples:
            e = math.fabs(fraction-lf)
            if e < error:
                error = e
                factor = f
        grid = factor * 10.**power
        if power > 3 or power < -3:
            format = '%+7.0e'
        elif power >= 0:
            digits = max(1, int(power))
            format = '%' + `digits`+'.0f'
        else:
            digits = -int(power)
            format = '%'+`digits+2`+'.'+`digits`+'f'
        ticks = []
        t = -grid*math.floor(-lower/grid)
        while t <= upper and len(ticks) < 200:
            #ticks.append(t, format % (t,))
            if form=='float':
                ticks.append((t,str(t)))
            else:
                ticks.append((int(t),str(int(t))))
            t = t + grid
        return ticks


    def _textBoundingBox(self, text):
        bg = self.draw.cget('background')
        dict = {'anchor': Tkinter.NW, 'text': text, 'fill': bg}
        if self.font is not None:
            dict['font'] = self.font
        item = apply(CanvasText, (self.draw, 0., 0.), dict)
        bb = self.draw.bbox(item)
        self.draw.delete(item)
        return bb



class GraphPoints:


    def __init__(self, points, attr):
        self.points = points
        self.scaled = self.points
        self.attributes = {}
        for name, value in self._attributes.items():
            try:
                value = attr[name]
            except KeyError: pass
            self.attributes[name] = value


    def boundingBox(self):
        return minBound(self.points),  maxBound(self.points)


    def fitToScale(self, scale=(1,1), shift=(0,0)):
        self.scaled = []
        for x,y in self.points:
            self.scaled.append(((scale[0]*x)+shift[0],\
                               (scale[1]*y)+shift[1]))
        self.anchor = scale[1]*self.attributes.get('anchor', 0.0)+\
                      shift[1]



class GraphLine(GraphPoints):


    def __init__(self, points, **attr):
        GraphPoints.__init__(self, points, attr)

    _attributes = {'color':       'black',
                   'width':        1,
                   'smooth':       0,
                   'splinesteps': 12}

    def draw(self, canvas):
        color  = self.attributes['color']
        width  = self.attributes['width']
        smooth = self.attributes['smooth']
        steps  = self.attributes['splinesteps']
        arguments = (canvas,)
        if smooth:
            for i in range(len(self.points)):
                x1, y1 = self.scaled[i]
                arguments = arguments + (x1, y1)
        else:
            for i in range(len(self.points)-1):
                x1, y1 = self.scaled[i]
                x2, y2 = self.scaled[i+1]
                arguments = arguments + (x1, y1, x2, y2)
        apply(Line, arguments, {'fill': color, 'width': width,
                                'smooth': smooth, 'splinesteps':steps})



class InteractiveGraphObject:


    def __init__(self,  **kw):
        self.id = None
        if kw.has_key('name'):
            self.name = kw['name']
            del kw['name']
        else:
            self.name = 'NoName'
        if kw.has_key('editor'):
            self.editor = kw['editor']
            del kw['editor']
        else:
            self.editor = None
        self.selected = 0


    def editNoneMenu(self, type, *args, **kw):
        apply(eval('self.menu'+type), args, kw)


    def __repr__(self):
        return '<%s %s>'%(self.__class__.__name__, self.name)


    def rename(self, name):
        self.name=name


    def select(self):
        self.selected = 1
        self.editor.draw.addtag_withtag('selected', self.iconTag)


    def deselect(self):
        self.selected = 0
        self.editor.draw.dtag(self.iconTag, 'selected')


    def delete(self):
        self.editor.draw.delete(self.iconTag)


    def gettags(self):
        self.editor.draw.gettags(self.iconTag)
        
        
    def instantiate(self, editor):
        """method to be implmented by subclass
        this method should create the geometric primitive for this item
        and tag them all with self.iconTag
        define its uniqueTag
        set its tags
        """
        self.editor = editor
        self.menu = Tkinter.Menu(editor, title = self.name)
        self.menu.add_command(label='info', command= self.showInfo)
        self.buildNodeIcon(editor)


    def buildNodeIcon(self, editor):
        print 'in buildNodeIcon'



class GraphObjects:


    def __init__(self, objects):
        self.objects = objects


    def boundingBox(self):
        c1, c2 = self.objects[0].boundingBox()
        for object in self.objects[1:]:
            c1o, c2o = object.boundingBox()
            c1 = minBound([c1, c1o])

            c2 = maxBound([c2, c2o])
        return c1, c2


    def fitToScale(self, scale=(1,1), shift=(0,0)):
        for object in self.objects:
            object.fitToScale(scale, shift)


    def draw(self, canvas):
        for object in self.objects:
            object.draw(canvas)



class HistogramNode(InteractiveGraphObject):


    def __init__(self, width=10, height=40, name='NoName',info=None,
                        editor=None, **kw):
        kw['name'] = name
        kw['editor'] = editor
        apply(InteractiveGraphObject.__init__,(self,), kw)
        self.width = width
        self.height = height
        #info is a link to state info
        self.info = info
        self.point = (width, height)
        self.instantiate(editor, info)


    def instantiate(self, editor, info=None):
        """method to be implmented by subclass
        this method should create the geometric primitive for this item
        and tag them all with self.iconTag
        define its uniqueTag
        set its tags
        """
        self.editor = editor
        self.info = info
        self.menu = Tkinter.Menu(editor, title = self.name)
        self.menu.add_command(label='info', command= self.showInfo)


    def showInfo(self, event=None):
        print 'in showInfo with ', self.info


    def buildNodeIcon(self, editor, x1, y1, x2, y2, fill='red',
            width=1, outline='black', stipple = ''):
        canvas = editor.draw
        g = canvas.create_rectangle(x1, y1, x2, y2, fill=fill,
                width=width, outline=outline, stipple=stipple)
        editor.geoms[g] = self
        canvas.lift(g)
        self.id = g



class HistogramNodeSet:


    def __init__(self, nodes, **kw):
        points = []
        self.nodes = nodes
        for n in nodes:
            points.append(n.point)
        self.points = points
        self.scaled = points
        self.attributes = {}
        #anchor is y axis of histogram base
        #spread is width of each bar
        self.attributes['anchor'] = 0
        self.attributes['spread'] = 5
        self.attributes['fill'] = 'blue'
        self.attributes['stipple'] = ''
        self.attributes['outline'] = 'black'
        self.attributes['width'] = 1
        for name, value in kw.items():
            self.attributes[name] = value


    def boundingBox(self):
        return minBound(self.points), maxBound(self.points)


    def fitToScale(self, scale=(1,1), shift=(0,0), p1=(1,1), p2=(1,1)):
        self.scaled = []
        for x, y in self.points:
            self.scaled.append(((scale[0]*x) + shift[0],\
                                (scale[1]*y) + shift[1]))
    
        #need bb[0]
        newanchor = scale[1]*p1[1]+ shift[1]
        self.attributes['anchor'] = newanchor


    def buildIcons(self, editor):
        fill = self.attributes['fill']
        width = self.attributes['width']
        stipple = self.attributes['stipple']
        outline = self.attributes['outline']
        spread = self.attributes['spread']
        anchor = self.attributes['anchor']
        canvas = editor.draw
        argument = (canvas,)
        p1, p2 = self.boundingBox()

        for i in range(len(self.nodes)):
            node = self.nodes[i]
            x1, y1 = self.scaled[i]
            node.buildNodeIcon(editor, x1-spread, y1, x1+spread,
                    anchor, fill=fill, width=width,
                    outline=outline, stipple=stipple)


                
class InteractiveHistogramGraph(InteractiveGraphBuilder):


    def __init__(self, name='InteractiveHistogramBuilder', master=None, 
               nodeList=[], yoffset=200, maxwidth=200, 
               reverseIndex=None, label_text=None,
               xlabel_text=None, ylabel_text=None, **kw):
        apply( InteractiveGraphBuilder.__init__,(self, name, master), kw)
        #then instantiate the nodes in nodeList
        #nodeList is list of tuples-> (width, height, info)
        self.spacing = 5
        self._multiples = [(2., math.log10(2.)), (5., math.log10(5.))] 
        self.reverseIndex = reverseIndex

        num = 0
        nodes = []
        #for w, h, info  in nodeList:
        for w, h  in nodeList:
            newname = '%d'%num
            newnode = HistogramNode(width=w, height=h,
                name=newname, info=num)
            self.nodes[newname] = newnode
            num = num + 1
        
        self.nodeSet = HistogramNodeSet(self.nodes.values())
        self._setsize()
        self.label_text = label_text
        self.xlabel_text = xlabel_text
        self.ylabel_text = ylabel_text
        self.buildIt(self.nodeSet, xaxis='automatic', yaxis='custom',
                ylabels='int')


if __name__== '__main__':
    root = Tkinter.Tk()
    #nodeList  = [(0,0,'origin'),(32, 20, 'first'), (15, 35, 'second'),(20, 10,'third')]
    #nodeList  = [(0,0),(32, 20 ), (15, 35 ),(20, 10,)]
    #histNB = InteractiveHistogramGraph('Test', master=root, nodeList=nodeList)
    import energy_histogram
    ehist = energy_histogram.EnergyHistogram()
    ehist.get_input_from_energies('ebind_top.hist')
    nodeList = ehist.histogram.array
    ehist.createReverseIndex()
    reverseIndex = ehist.reverseIndex
    histNB = InteractiveHistogramGraph('test', master=root, nodeList=nodeList,
        reverseIndex = reverseIndex)
    

#from InteractiveGraph import InteractiveHistogramGraph
#import energy_histogram
#ehist = energy_histogram.EnergyHistogram()
#ehist.get_input_from_energies('ebind_top.hist')
#nodeList = ehist.histogram.array
#import Tkinter
#root = Tkinter.Tk()
#histNB = InteractiveHistogramGraph('test', master=root, nodeList=nodeList)






########################################################################
#
#    Vision Macro - Python source code - file generated by vision
#    Thursday 22 July 2010 11:38:41 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/VisionInterface/Adt/Macro/TrajQR.py,v 1.1 2010/07/22 18:43:31 jren Exp $
#
# $Id: TrajQR.py,v 1.1 2010/07/22 18:43:31 jren Exp $
#

from NetworkEditor.macros import MacroNode
from NetworkEditor.macros import MacroNode
class TrajQR(MacroNode):
    '''
    This node runs Traj QR.

    Inputs:
       port 1: directory containing pdb files
       port 2: rmsd
       port 3: directory to copy the selected pdb files to

    Output:
       directory that containst he selected pdb files
    '''

    def __init__(self, constrkw={}, name='TrajQR', **kw):
        kw['name'] = name
        apply( MacroNode.__init__, (self,), kw)

    def beforeAddingToNetwork(self, net):
        MacroNode.beforeAddingToNetwork(self, net)
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        net.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")
        from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
        try:
            addOpalServerAsCategory("http://kryptonite.nbcr.net/opal2", replace=False)
        except:
            pass

    def afterAddingToNetwork(self):
        masterNet = self.macroNetwork
        from NetworkEditor.macros import MacroNode
        MacroNode.afterAddingToNetwork(self)
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        ## building macro network ##
        TrajQR_0 = self
        from traceback import print_exc
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        masterNet.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")
        from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
        try:
            addOpalServerAsCategory("http://kryptonite.nbcr.net/opal2", replace=False)
        except:
            pass
        try:
            ## saving node input Ports ##
            input_Ports_1 = self.macroNetwork.ipNode
            apply(input_Ports_1.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore MacroInputNode named input Ports in network self.macroNetwork"
            print_exc()
            input_Ports_1=None

        try:
            ## saving node output Ports ##
            output_Ports_2 = self.macroNetwork.opNode
            apply(output_Ports_2.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            output_Ports_2.move(183, 405)
        except:
            print "WARNING: failed to restore MacroOutputNode named output Ports in network self.macroNetwork"
            print_exc()
            output_Ports_2=None

        try:
            ## saving node Make_Zip_File ##
            from Vision.StandardNodes import MakeZipFileNE
            Make_Zip_File_3 = MakeZipFileNE(constrkw={}, name='Make_Zip_File', library=stdlib)
            self.macroNetwork.addNode(Make_Zip_File_3,47,75)
            apply(Make_Zip_File_3.inputPortByName['input_directory'].configure, (), {'defaultValue': None})
            apply(Make_Zip_File_3.inputPortByName['output_directory'].configure, (), {'defaultValue': None})
            apply(Make_Zip_File_3.inputPortByName['output_name'].configure, (), {'defaultValue': None})
            apply(Make_Zip_File_3.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore MakeZipFileNE named Make_Zip_File in network self.macroNetwork"
            print_exc()
            Make_Zip_File_3=None

        try:
            ## saving node TrajQR_kryptonite_nbcr_net ##
            from NetworkEditor.items import FunctionNode
            TrajQR_kryptonite_nbcr_net_4 = FunctionNode(functionOrString='TrajQR_kryptonite_nbcr_net', host="http://kryptonite.nbcr.net/opal2", namedArgs={'license': False, 'rmsd': '', 'qh_filename': 'qh.tre', 'qr_filename': 'qr.out', 'compressed_pdbs': '', 'localRun': False, 'execPath': ''}, constrkw={'functionOrString': "'TrajQR_kryptonite_nbcr_net'", 'host': '"http://kryptonite.nbcr.net/opal2"', 'namedArgs': {'license': False, 'rmsd': '', 'qh_filename': 'qh.tre', 'qr_filename': 'qr.out', 'compressed_pdbs': '', 'localRun': False, 'execPath': ''}}, name='TrajQR_kryptonite_nbcr_net', library=wslib)
            self.macroNetwork.addNode(TrajQR_kryptonite_nbcr_net_4,279,135)
            apply(TrajQR_kryptonite_nbcr_net_4.inputPortByName['license'].configure, (), {'defaultValue': None})
            apply(TrajQR_kryptonite_nbcr_net_4.inputPortByName['rmsd'].configure, (), {'defaultValue': None, 'required': True})
            apply(TrajQR_kryptonite_nbcr_net_4.inputPortByName['qh_filename'].configure, (), {'defaultValue': None})
            apply(TrajQR_kryptonite_nbcr_net_4.inputPortByName['qr_filename'].configure, (), {'defaultValue': None})
            apply(TrajQR_kryptonite_nbcr_net_4.inputPortByName['compressed_pdbs'].configure, (), {'defaultValue': None, 'required': True})
            apply(TrajQR_kryptonite_nbcr_net_4.inputPortByName['localRun'].configure, (), {'defaultValue': None})
            apply(TrajQR_kryptonite_nbcr_net_4.inputPortByName['execPath'].configure, (), {'defaultValue': None})
            TrajQR_kryptonite_nbcr_net_4.inputPortByName['license'].widget.set(0, run=False)
            TrajQR_kryptonite_nbcr_net_4.inputPortByName['rmsd'].rebindWidget()
            TrajQR_kryptonite_nbcr_net_4.inputPortByName['rmsd'].widget.set(r"", run=False)
            TrajQR_kryptonite_nbcr_net_4.inputPortByName['rmsd'].unbindWidget()
            TrajQR_kryptonite_nbcr_net_4.inputPortByName['qh_filename'].widget.set(r"qh.tre", run=False)
            TrajQR_kryptonite_nbcr_net_4.inputPortByName['qr_filename'].widget.set(r"qr.out", run=False)
            TrajQR_kryptonite_nbcr_net_4.inputPortByName['compressed_pdbs'].rebindWidget()
            TrajQR_kryptonite_nbcr_net_4.inputPortByName['compressed_pdbs'].widget.set(r"", run=False)
            TrajQR_kryptonite_nbcr_net_4.inputPortByName['compressed_pdbs'].unbindWidget()
            TrajQR_kryptonite_nbcr_net_4.inputPortByName['localRun'].widget.set(0, run=False)
            TrajQR_kryptonite_nbcr_net_4.inputPortByName['execPath'].widget.set(r"", run=False)
            apply(TrajQR_kryptonite_nbcr_net_4.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore FunctionNode named TrajQR_kryptonite_nbcr_net in network self.macroNetwork"
            print_exc()
            TrajQR_kryptonite_nbcr_net_4=None

        try:
            ## saving node SelectOnExtension ##
            from Vision.StandardNodes import SelectOnExtension
            SelectOnExtension_5 = SelectOnExtension(constrkw={}, name='SelectOnExtension', library=stdlib)
            self.macroNetwork.addNode(SelectOnExtension_5,279,192)
            apply(SelectOnExtension_5.inputPortByName['filenames'].configure, (), {'defaultValue': None})
            apply(SelectOnExtension_5.inputPortByName['extension'].configure, (), {'defaultValue': None})
            SelectOnExtension_5.inputPortByName['extension'].widget.set(r".out", run=False)
            apply(SelectOnExtension_5.configure, (), {'paramPanelImmediate': 1})
        except:
            print "WARNING: failed to restore SelectOnExtension named SelectOnExtension in network self.macroNetwork"
            print_exc()
            SelectOnExtension_5=None

        try:
            ## saving node Index ##
            from Vision.StandardNodes import Index
            Index_6 = Index(constrkw={}, name='Index', library=stdlib)
            self.macroNetwork.addNode(Index_6,123,281)
            apply(Index_6.inputPortByName['data'].configure, (), {'datatype': 'list', 'defaultValue': None, 'originalDatatype': 'None'})
            apply(Index_6.inputPortByName['index'].configure, (), {'defaultValue': None})
            apply(Index_6.outputPortByName['data'].configure, (), {'datatype': 'string'})
            apply(Index_6.inputPortByName['index'].widget.configure, (), {'max': 0, 'min': -1})
            Index_6.inputPortByName['index'].widget.set(0, run=False)
            apply(Index_6.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore Index named Index in network self.macroNetwork"
            print_exc()
            Index_6=None

        try:
            ## saving node GetSelectedPDBs ##
            from Vision.StandardNodes import Generic
            GetSelectedPDBs_7 = Generic(constrkw={}, name='GetSelectedPDBs', library=stdlib)
            self.macroNetwork.addNode(GetSelectedPDBs_7,200,346)
            apply(GetSelectedPDBs_7.addInputPort, (), {'singleConnection': True, 'name': 'trajqr_url', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
            apply(GetSelectedPDBs_7.addInputPort, (), {'singleConnection': True, 'name': 'pdb_in_dir', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
            apply(GetSelectedPDBs_7.addInputPort, (), {'singleConnection': True, 'name': 'pdb_out_dir', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
            apply(GetSelectedPDBs_7.addOutputPort, (), {'name': 'pdb_out_dir', 'datatype': 'string', 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
            code = """def doit(self, trajqr_url, pdb_in_dir, pdb_out_dir):
        pdb_in_dir = os.path.abspath(pdb_in_dir)
        pdb_out_dir = os.path.abspath(pdb_out_dir)
        import urllib
        import shutil
        pdb_list = []

        f = urllib.urlopen(trajqr_url)

        for i in f.readlines():
            pdb_file = i.strip('''
''')+'''.pdb'''
            infile = pdb_in_dir + os.sep + pdb_file
            outfile = pdb_out_dir + os.sep + pdb_file
            shutil.copyfile(infile, outfile)
        
        f.close
	pass
## to ouput data on port pdb_list use
        self.outputData(pdb_out_dir=pdb_out_dir)


"""
            GetSelectedPDBs_7.configure(function=code)
            apply(GetSelectedPDBs_7.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore Generic named GetSelectedPDBs in network self.macroNetwork"
            print_exc()
            GetSelectedPDBs_7=None

        #self.macroNetwork.run()
        self.macroNetwork.freeze()

        ## saving connections for network TrajQR ##
        input_Ports_1 = self.macroNetwork.ipNode
        if input_Ports_1 is not None and Make_Zip_File_3 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, Make_Zip_File_3, "new", "input_directory", blocking=True
                    , splitratio=[0.67469172328672133, 0.50941480178765064])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and Make_Zip_File_3 in network self.macroNetwork"
        if input_Ports_1 is not None and TrajQR_kryptonite_nbcr_net_4 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, TrajQR_kryptonite_nbcr_net_4, "new", "rmsd", blocking=True
                    , splitratio=[0.45681934134493246, 0.33803410340720857])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and TrajQR_kryptonite_nbcr_net_4 in network self.macroNetwork"
        if Make_Zip_File_3 is not None and TrajQR_kryptonite_nbcr_net_4 is not None:
            try:
                self.macroNetwork.connectNodes(
                    Make_Zip_File_3, TrajQR_kryptonite_nbcr_net_4, "zipfile", "compressed_pdbs", blocking=True
                    , splitratio=[0.71081093868178202, 0.43908686250860129])
            except:
                print "WARNING: failed to restore connection between Make_Zip_File_3 and TrajQR_kryptonite_nbcr_net_4 in network self.macroNetwork"
        if TrajQR_kryptonite_nbcr_net_4 is not None and SelectOnExtension_5 is not None:
            try:
                self.macroNetwork.connectNodes(
                    TrajQR_kryptonite_nbcr_net_4, SelectOnExtension_5, "result", "filenames", blocking=True
                    , splitratio=[0.63533198718963335, 0.2560040659014205])
            except:
                print "WARNING: failed to restore connection between TrajQR_kryptonite_nbcr_net_4 and SelectOnExtension_5 in network self.macroNetwork"
        if SelectOnExtension_5 is not None and Index_6 is not None:
            try:
                self.macroNetwork.connectNodes(
                    SelectOnExtension_5, Index_6, "matching", "data", blocking=True
                    , splitratio=[0.65832290589891362, 0.21624347250969889])
            except:
                print "WARNING: failed to restore connection between SelectOnExtension_5 and Index_6 in network self.macroNetwork"
        if Index_6 is not None and GetSelectedPDBs_7 is not None:
            try:
                self.macroNetwork.connectNodes(
                    Index_6, GetSelectedPDBs_7, "data", "trajqr_url", blocking=True
                    , splitratio=[0.51754619993986595, 0.63949220525823236])
            except:
                print "WARNING: failed to restore connection between Index_6 and GetSelectedPDBs_7 in network self.macroNetwork"
        if input_Ports_1 is not None and GetSelectedPDBs_7 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, GetSelectedPDBs_7, "Make_Zip_File_input_directory", "pdb_in_dir", blocking=True
                    , splitratio=[0.7121964126696847, 0.42697952864467786])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and GetSelectedPDBs_7 in network self.macroNetwork"
        if input_Ports_1 is not None and GetSelectedPDBs_7 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, GetSelectedPDBs_7, "new", "pdb_out_dir", blocking=True
                    , splitratio=[0.62460995639428174, 0.60275185701976297])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and GetSelectedPDBs_7 in network self.macroNetwork"
        output_Ports_2 = self.macroNetwork.opNode
        if GetSelectedPDBs_7 is not None and output_Ports_2 is not None:
            try:
                self.macroNetwork.connectNodes(
                    GetSelectedPDBs_7, output_Ports_2, "pdb_out_dir", "new", blocking=True
                    , splitratio=[0.7431176169343523, 0.73697976585137526])
            except:
                print "WARNING: failed to restore connection between GetSelectedPDBs_7 and output_Ports_2 in network self.macroNetwork"
        self.macroNetwork.runOnNewData.value = True

        ## modifying MacroInputNode dynamic ports
        input_Ports_1 = self.macroNetwork.ipNode
        input_Ports_1.outputPorts[1].configure(name='Make_Zip_File_input_directory')
        input_Ports_1.outputPorts[2].configure(name='TrajQR_kryptonite_nbcr_net_rmsd')
        input_Ports_1.outputPorts[3].configure(name='GetSelectedPDBs_pdb_out_dir')

        ## modifying MacroOutputNode dynamic ports
        output_Ports_2 = self.macroNetwork.opNode
        output_Ports_2.inputPorts[1].configure(singleConnection='auto')
        output_Ports_2.inputPorts[1].configure(name='GetSelectedPDBs_pdb_out_dir')
        ## configure MacroNode input ports
        TrajQR_0.inputPorts[0].configure(name='Make_Zip_File_input_directory')
        TrajQR_0.inputPorts[0].configure(datatype='string')
        TrajQR_0.inputPorts[1].configure(name='TrajQR_kryptonite_nbcr_net_rmsd')
        TrajQR_0.inputPorts[1].configure(datatype='string')
        TrajQR_0.inputPorts[2].configure(name='GetSelectedPDBs_pdb_out_dir')
        TrajQR_0.inputPorts[2].configure(datatype='string')
        ## configure MacroNode output ports
        TrajQR_0.outputPorts[0].configure(name='GetSelectedPDBs_pdb_out_dir')
        TrajQR_0.outputPorts[0].configure(datatype='string')

        TrajQR_0.shrink()

        ## reset modifications ##
        TrajQR_0.resetTags()
        TrajQR_0.buildOriginalList()



#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/write_conformations_from_dlg.py,v 1.5 2014/03/25 17:07:02 rhuey Exp $
#
# $Id: write_conformations_from_dlg.py,v 1.5 2014/03/25 17:07:02 rhuey Exp $
#
import os

from AutoDockTools.Docking import Docking


if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: write_conformations_from_dlg.py -d directory"
        print
        print "    Description of command..."
        print "         -d     docking filename"
        print "    Optional parameters:"
        print "        [-o]    output file stem"
        print "        (default is ligandname. Outputfiles are named 'stem' plus '_num.pdbq[t]')"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'd:o:vh')
    except getopt.GetoptError, msg:
        print 'write_conformations_from_dlg.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-d: docking log filename
    docking_filename =  None

    # initialize optional parameter
    #-o output_stem
    output_stem = None
    #-v verbose best only
    verbose = False

    #'d:bvh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-d', '--d'):
            docking_filename = a
            if verbose: print 'set docking_filename to ', a
        if o in ('-o', '--o'):
            output_stem = a
            if verbose: print 'set output_stem to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if not docking_filename:
        print 'write_conformations_from_dlg: docking_filename must be specified.'
        usage()
        sys.exit()


    d = Docking()
    d.readDlg(docking_filename)
    lines = d.ligMol.parser.allLines
    for i in range(len(lines)):
        line = lines[i] 
        if line.find("\n")==-1:
            d.ligMol.parser.allLines[i] = line + "\n"
    ext = '.pdbq'
    if d.version>=4.0:
        ext = '.pdbqt'
        
    if output_stem is None:
        output_stem = d.ligMol.name

    for ix, conf in enumerate(d.ch.conformations):
        outfile= output_stem + '_' + str(ix+1) + ext
        conf = d.ch.conformations[ix]
        d.ligMol.parser.write_with_new_coords(conf.getCoords(),outfile)
        if verbose: print "wrote ", outfile


# To execute this command type:
# write_conformations_from_dlg.py -d docking_filename 
# optional arguments
# -o outputfile_stem (default is ligandname)



#############################################################################
#
# Author: Ruth HUEY, Michel F. SANNER
#
# Copyright: M. Sanner TSRI 2000
#
#############################################################################

#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/autotors4Commands.py,v 1.3 2009/02/10 17:44:05 rhuey Exp $
#
# $Id: autotors4Commands.py,v 1.3 2009/02/10 17:44:05 rhuey Exp $
#
#
#
#
#
#
#
"""
This Module facilitates selecting and formatting a ligand for a subsequent 
AutoDock4.0 calculation.  The steps in this process are:

    * The user selects the small molecule from a list of molecules 
already in the moleculeViewer OR as a PDBQ file, a PDB file or
a MOL2 file from a fileBrowser.  

    * The user selects the ROOT atom of the ligand either: 

        o     by picking it or 

        o     by autoroot which sets the root to be the atom in the 
            molecule which has the smallest 'largest sub-tree.'

    * Next the user decides which possible and active torsions he wants 
to disallow, changing them from active to inactive. This is done by picking 
an active 'green' bond which turns it inactive or 'purple'. This is 
reversible. The user can also disallow all peptide backbone torsions and/or 
all torsions of amide bonds.

    * Carbons in cycles can be tested for aromaticity.  If the angle 
between the normals to adjacent atoms in the cycle is less than 7.5 Degrees, 
the cycle is considered aromatic: its carbons are renamed "A.." and their 
element type set to 'A'. (This is for the force-field calculations done 
in AutoDock.) This Module does this conversion reversibly. Also, the user 
is able to select a carbon to convert (reversibly) and he can change the
the value of the aromaticity cut-off.

    * Non-polar hydrogens and lone pairs are merged which means that the charge of 
each is added to its heavy atom and the hydrogen atoms themselves are not written 
in the output file, thus in some sense 'removing' them from the molecule. 
'Fewer' atoms simplifies the AutoDock run.

    * The last function of this Module is to write a file which contains 
the correctly formatted ligand atoms.  The ROOT section of the molecule 
expands from the selected ROOT atom out to include all atoms adjacent to it 
up to the first active torsion.  The active torsions set the position of 
BRANCH key words in the output pdbq file (and their corresponding 
ENDBRANCH  key words). These keywords are nested to set up  a 
Breadth-First Order Traversal.  Autotors also calculates the torsional degrees 
of freedom (TORSDOF) which is the number of possible torsions less the number of 
symmetry-equivalent torsions (such as a bond to a NH3). This key word is the 
last line of the pdbq file. 
"""
import Tkinter
from ViewerFramework.VFCommand import CommandGUI

from AutoDockTools.autotorsCommands import rootSph, markSph,\
menuText, AtorsMoleculeChooser, MAXTORS, AdtSetMode,\
AtorsReader, Ators4MoleculeChooser, Ators4Reader, AtorsRefWriter, \
RigidMolecule, RigidMolecule4, AUTOTORSWriter, AUTOTORS4Writer, \
MarkRoot, SelectRoot, SetTorsionNumberGUICommand, SetTorsionNumber, \
AutoRoot, SetRotatableBonds, DefiningRotatableBonds, SetBondRotatableFlag,\
CheckAromatic, StopCheckAromatic, SetCarbonNames, ChangeAromaticCutOff, \
TogglerootSphere, AutoAutoTors, StopAutoTors, AtorsInit, AtorsInitMol, \
ProcessCharges, ProcessBonds, rootSph, markSph, check_autotors_geoms,\
MAXTORS, menuText, warningText, checkMolCharges,\
autoMergeNPHS, set_autoMergeNPHS


Ators4MoleculeChooserGUI=CommandGUI()
Ators4MoleculeChooserGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'], menuText['Choose Molecule4'], cascadeName = menuText['Input Molecule'])


Ators4ReaderGUI = CommandGUI()
Ators4ReaderGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'], menuText['Read Molecule4'], cascadeName = menuText['Input Molecule'])


AtorsRefWriterGUI = CommandGUI()
AtorsRefWriterGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'], menuText['Ref Molecule'], cascadeName = menuText['Input Molecule'])


RigidMolecule4GUI = CommandGUI()
RigidMolecule4GUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'], menuText['Rigid Molecule4'], cascadeName = menuText['Input Molecule'])


AUTOTORS4WriterGUI=CommandGUI()
AUTOTORS4WriterGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'], menuText['WritePDBQTMB'],
            cascadeName = menuText['WriteMB'])


MarkRootGUI=CommandGUI()
MarkRootGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'], menuText['SRA1'],
            cascadeName = menuText['DefineRigidRootMB'])


SelectRootGUI=CommandGUI()
SelectRootGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'],\
 menuText['ByPicking'], cascadeName = menuText['DefineRigidRootMB'])


SetTorsionNumberGUICommandGUI=CommandGUI()
SetTorsionNumberGUICommandGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'],\
 menuText['SetTorsionNumber'], cascadeName = menuText['DefineRigidRootMB'] )



AutoRootGUI=CommandGUI()
AutoRootGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'],\
menuText['Automatically'], cascadeName = menuText['DefineRigidRootMB'])


DefiningRotatableBondsGUI = CommandGUI()
DefiningRotatableBondsGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'],\
    menuText['DefineRotatableBonds'], cascadeName = menuText['DefineRigidRootMB'])


CheckAromaticGUI = CommandGUI()
CheckAromaticGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'],\
menuText['RenameAromaticCarbons'], cascadeName = menuText['AromaticCarbonsMB'])


StopCheckAromaticGUI = CommandGUI()
StopCheckAromaticGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'],\
menuText['RestoreAliphaticCarbons'], cascadeName = menuText['AromaticCarbonsMB'])


SetCarbonNamesGUI = CommandGUI()
SetCarbonNamesGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'],\
menuText['SetCarbonNames'], cascadeName = menuText['AromaticCarbonsMB'])


ChangeAromaticCutOffGUI=CommandGUI()
ChangeAromaticCutOffGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'],\
menuText['ChangeAromaticityCriteria'], cascadeName = menuText['AromaticCarbonsMB'])


TogglerootSphereGUI=CommandGUI()
TogglerootSphereGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'],\
menuText['ShowAutotorsRootSphMB'], cascadeName = menuText['DefineRigidRootMB'])


AutoAutoTorsGUI=CommandGUI()
AutoAutoTorsGUI.addMenuCommand('AutoTools4Bar', menuText['AutoTorsMB'], \
menuText['AutomaticAutotorsSetupMB'], cascadeName = menuText['Input Molecule'])



commandList = [
    
    {'name':'AD4tors_readLigand','cmd':Ators4Reader(),
        'gui':Ators4ReaderGUI},
    {'name':'AD4tors_chooseLigand','cmd':Ators4MoleculeChooser(),
        'gui':Ators4MoleculeChooserGUI},
    {'name':'AD4tors_rigidLigand','cmd':RigidMolecule4(),
        'gui':RigidMolecule4GUI},
    {'name':'AD4tors_automaticLigandFormatting','cmd':AutoAutoTors(),
        'gui':AutoAutoTorsGUI},
#    {'name':'AD4tors_writeRef','cmd':AtorsRefWriter(),
#        'gui':AtorsRefWriterGUI},
    {'name':'AD4tors_setRoot','cmd':SelectRoot(),'gui':SelectRootGUI},
    {'name':'AD4tors_autoRoot','cmd':AutoRoot(),'gui':AutoRootGUI},
#    {'name':'AD4tors_addChainToRootGC','cmd':AddChainToRootGUICommand(),
#        'gui':AddChainToRootGUICommandGUI},
#    {'name':'AD4tors_addChainToRoot','cmd':AddChainToRoot(),'gui':None},
#    {'name':'AD4tors_removeChainFromRootGC','cmd':RemoveChainFromRootGUICommand(),
#        'gui':RemoveChainFromRootGUICommandGUI},
#    {'name':'AD4tors_removeChainFromRoot','cmd':RemoveChainFromRoot(),'gui':None},
    {'name':'AD4tors_markRoot','cmd':MarkRoot(),'gui':MarkRootGUI},
    {'name':'AD4tors_showRootSphere','cmd':TogglerootSphere(),
        'gui':TogglerootSphereGUI},

    {'name':'AD4tors_defineRotBonds', 'cmd':DefiningRotatableBonds(),
         'gui':DefiningRotatableBondsGUI },
    {'name':'AD4tors_limitTorsionsGC','cmd':SetTorsionNumberGUICommand(),
        'gui':SetTorsionNumberGUICommandGUI},
##    {'name':'AD4tors_changePlanarCarbonsToA','cmd':CheckAromatic(),
##      'gui':CheckAromaticGUI},
##  {'name':'AD4tors_changeAromaticCarbonsToC','cmd':StopCheckAromatic(),
##      'gui':StopCheckAromaticGUI},
    {'name':'AD4tors_setCarbonNames','cmd':SetCarbonNames(),
        'gui':SetCarbonNamesGUI},
    {'name':'AD4tors_changePlanarityCriteria','cmd':ChangeAromaticCutOff(),
        'gui':ChangeAromaticCutOffGUI},
    {'name':'AD4tors_writeFormattedPDBQT','cmd':AUTOTORS4Writer(),
        'gui':AUTOTORS4WriterGUI},
    ]


def initModule(vf):
    for dict in commandList:
        vf.addCommand(dict['cmd'],dict['name'],dict['gui'])
    if not hasattr(vf, 'ADTSetMode'):
        vf.addCommand(AdtSetMode(), 'ADTSetMode')
    if not hasattr(vf, 'ADtors_limitTorsions'):
        vf.addCommand(SetTorsionNumber(), 'ADtors_limitTorsions')
    if not hasattr(vf, 'ADtors_setBondRotatableFlag'):
        vf.addCommand(SetBondRotatableFlag(), 'ADtors_setBondRotatableFlag')
    if not hasattr(vf, 'ADtors_stop'):
        vf.addCommand(StopAutoTors(), 'ADtors_stop')

    if vf.hasGui:
        vf.GUI.menuBars['AutoTools4Bar']._frame.config( {'background':'tan'})
        for item in vf.GUI.menuBars['AutoTools4Bar'].menubuttons.values():
            item.configure(background = 'tan')
        if not hasattr(vf.GUI, 'adt4Bar'):
            vf.GUI.adt4Bar = vf.GUI.menuBars['AutoTools4Bar']
            vf.GUI.adt4Frame = vf.GUI.adt4Bar.menubuttons.values()[0].master
        if not hasattr(vf.GUI, 'adt4ModeLabel'):
            mbs = {}
            packing_list = []
            for c in vf.GUI.adt4Frame.children.values():
                if isinstance(c, Tkinter.Menubutton):
                    mbs[c.cget('text')] = c
                    packing_list.append(c.cget('text'))
                    c.pack_forget()
            vf.GUI.adt4ModeLabel=Tkinter.Label(vf.GUI.adt4Frame, text="ADT4.0", width=6,
                             relief='sunken', borderwidth=1, fg='DarkGreen',
                             bg = 'ivory',anchor='w' )
            vf.GUI.adt4ModeLabel.pack(side='left')
            vf.GUI.adt4ModeLabel.bind("<Double-Button-1>", vf.ADTSetMode.guiCallback())
            for t in packing_list:
                try:
                    c = mbs[t]
                    c.pack(side='left')
                except:
                    pass

#        if not hasattr(vf.GUI, 'ligandLabel'):
#            vf.GUI.ligandLabelLabel = Tkinter.Label(vf.GUI.adtFrame, \
#                            text="Ligand:", bg='tan')
#            vf.GUI.ligandLabelLabel.pack(side='left')
#            vf.GUI.ligandLabel=Tkinter.Label(vf.GUI.adtFrame, text="None", width=4,
#                                     relief='sunken', borderwidth=1,
#                                     anchor='w' )
#            vf.GUI.ligandLabel.pack(side='left')


